/* File name     : cc_common.h
 * Creation date : July 1992
 * Modified date : Feburary 1993
 * Author        : Y.Yasu, Online group, Physics department, KEK
 */

/* CAMAC status */
#define CC_STA_OK               0
#define CC_STA_SINGLE_TIMEOUT   -1
#define CC_STA_BLOCK_INVFUNC    -2
#define CC_STA_BLOCK_TIMEOUT    -3
#define CC_STA_LIST_STOP        -10
#define CC_STA_LIST_TIMEOUT     -11
#define CC_STA_LIST_EMPTY       -12
#define CC_STA_LIST_ZERODIV     -13
#define CC_STA_LIST_OVERSTEP    -14
#define CC_STA_KLIST_SAMENAME   -21
#define CC_STA_KLIST_MEMORYFULL -22
#define CC_STA_KLIST_NONAME     -23

/* ioctl command */
#define CCIOC_WAIT_LAM          1 /* wait LAM interrupt */

#define CC_K_MAX_BRANCH 1
#define CC_K_MAX_CRATE  8

/* NAF code generation */
#define NAFGEN(n,a,f) ((((n<<4) + a) << 5) + f)

/*
 Description of status
   There are two types of status, system status and camac status.
   The system status means return camlib function value while the camac status
   means the return value of status of iosb. For examples, CC_STA_OK,
   CC_STA_SINGLE_TIMEOUT,... are the camac status.  And, ENXIO, EBUSY,.. are
   the system status which is defined in the files of the directory
   /usr/include.
   Therefore,
   system status = 0 and camac status = 0 : all are ok.
   system status = -1 and camac status !=0 : system call is ok, but camac fails
       : or list operation fails.
   system status !=0 except -1            : system call is bad.
 * status :
   camac status  : success = 0 while error != 0. The error is negative value
   which is defined in this file.
 * ret_length :
   actual length of data transferred in short word
 * s_register :
   Only a parameter which is passed to device driver.
   latest contents of s register is return
 * devinfo
    first short word  : Control Status Register(0:15)
    second short word : Channel Status/Error Register(0:4)
*/
struct cc_iosb {
    int status;
    int ret_length;
    int s_reg;
    int devinfo;
  };

struct camcmd {
    short cmdadr;
    short cmd;
    short naf;
    short wc;
    short dummy;
    short camhalt;
  };

struct reg2917s {
    u_short cser;
    u_short docr;
    u_short sccr;
    u_short mtcr;
    u_short machi;
    u_short maclo;
    u_short lamicr;
    u_short donicr;
    u_short empicr;
    u_short aboicr;
    u_short lamivr;
    u_short donivr;
    u_short empivr;
    u_short aboivr;
    u_short cmar;
    u_short cwcr;
    u_short srr;
    u_short csr;
};
