/*
 * analyzer_fast.c
 *                   Simple analyzer for nova development
 *              version 1.02  01-OCT-1992 M. Nomachi (KEK)
 *
 * usage: analyzer_fast [priority [name]]]
 */
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "nova.h"

#define DEFAULT_NAME     "/tmp/nova"
#define DEFAULT_PRI      50

int buffer[66000];

main(argc, argv) int argc; char *argv[]; {
  struct NOVAD *nd, *nova_copen();
  char name[NV_MAXNAME + 1];
  int *data, length, priority;
  int status, loop, i, sleep_time;
  int get_count, skip_count;
  
  /* Read options */
  priority   = (argc > 1) ? atoi(argv[1]) : DEFAULT_PRI;
  strcpy(name, (argc > 2) ? argv[2] : DEFAULT_NAME);
  
  /* open NOVA */
  if ((int)(nd = nova_open(name, priority,1)) == -1) {
    printf("analyzer: cannot open %s", name);
    exit(1);
  }
  
  get_count = 0;
  skip_count = 0;
  loop = 0;
  while(1) {
    alarm(10);
    status = nova_get(nd, &data, &length);
    switch (status) {
    case 0:   /* nomal */
      skip_count += (data[0]-loop-1);
      loop = data[0];
      get_count++;
      for(i = 0; i < (length>>2); i++) buffer[i]=data[i];
      status = nova_put(nd, data, 0, 0);
      break;
    case 1:   /* Interrupted */
      if (get_count+skip_count) 
	printf("analyzer: interrupted! get %d / %d %d%%\n",
	       get_count, get_count+skip_count,
	       100*get_count/(get_count+skip_count));
      else
	printf("analyzer: interrupted! get %d / %d \n",
	       get_count, get_count+skip_count);
      get_count = 0;
      skip_count = 0;
      loop = 0;
      break;
    default:  /* error */
      printf("analyzer: error! %d\n",status);
      break;
    }
    if (status < 0)  break;
  }
  
  printf("analyzer: nomal end.\n");
  nova_close(nd);
}

