/*
 * novand.c
 *
 * Nova buffer manager network daemon program
 *
 *              version 1.03  08-OCT-1992 Masaharu NOMACHI   (KEK online)
 *              version 1.08  18-JAN-1994 Yasuo TAKEUCHI     (T.I.T.)
 */

#include "novanet.h"
#ifdef VXWORKS
void txproc(), rxproc();
#endif

char unknown[] = {"unknown"};

#ifndef VXWORKS
#ifdef SGI
int child_end();
#else
void child_end();
#endif
struct sigvec vec;
#endif

#ifdef VXWORKS
novand_main(argv1,argv2,argv3,argv4,argv5,argv6,argv7,argv8,argv9,
  argv10)
char *argv1,*argv2,*argv3,*argv4,*argv5,*argv6,*argv7,*argv8,*argv9,*argv10;
{
  char *argv[] = {NULL,argv1,argv2,argv3,argv4,argv5,argv6,argv7,argv8,argv9,
                  argv10};
  int argc;
#else
main(argc,argv) int argc; char **argv; 
{
#endif
  int sock, length, port;
  struct sockaddr_in server, cliant;
  struct hostent *hp;
  int msgsock, rval;
#ifdef VXWORKS
  char vhname[100];
  void txproc(), rxproc();
  int i;
#endif
  char *name;
  int status;
  int pid;

#ifdef VXWORKS
   argv[0] = "novand";
   for(i=0;i<11;i++) {
     if(argv[i] == NULL || *argv[i] == '\0') {
       argc = i;
       break;
     }
   }
#endif

  /* Create socket */
  sock = socket(AF_INET, SOCK_STREAM, 0);
  if (sock < 0) {
    printf("** ERROR ** opening stream socket. errno=%X\n",errno);
    exit(1);
  }

  /* Name socket using wildcards */
  server.sin_family = AF_INET;
  server.sin_addr.s_addr = INADDR_ANY;
#ifdef VXWORKS
  if (argv[1] != NULL)  port = atoi(argv[1]);
  else                  port = NOVA_PORT;
  server.sin_port = port;
#else
  if (argc>1)           port = atoi(argv[1]);
  else                  port = NOVA_PORT;
  server.sin_port = htons(port);
#endif
  if (bind(sock, (struct sockaddr *)&server, sizeof(server)) == -1) {
    printf("** ERROR ** binding stream socket. errno=%X",errno);
    exit(1);
  }

  /* Find out assigned port number and print it out */
  length = sizeof(server);
  if (getsockname(sock, (struct sockaddr *)&server, &length) == -1) {
    printf("** ERROR ** getting socket name. errno=%X\n",errno);
    exit(1);
  }
#ifdef VXWORKS
  printf("Socket %d has port #%d\n",sock,server.sin_port);
#else
  printf("Socket %d has port #%d\n",sock,ntohs(server.sin_port));
#endif

  /* add signal handler */
#ifndef VXWORKS
#if defined(SGI)
  vec.sv_handler = (int (*)())child_end;
#else
  vec.sv_handler = (void (*)())child_end;
#endif
  vec.sv_mask = 0;
  vec.sv_flags = 0;
#endif

#ifdef HP
  sigvector(SIGCHLD, &vec, (struct sigvec *) NULL);
#else
#if !defined(VXWORKS)
  sigvec(SIGCHLD, &vec, (struct sigvec *) NULL);
#endif
#endif

  /* Start accepting connections */
  status = listen(sock,5);
  if (status == -1) {
    printf(" ***** listen ****** errno = %X\n",errno);
    exit(1);
  }

  do {
      length = sizeof(cliant);
      msgsock = accept(sock,(struct sockaddr *)&cliant,&length);
      if (msgsock >= 0) {
#ifdef VXWORKS
	  if (hostGetByAddr(cliant.sin_addr.s_addr,vhname) == ERROR) {
	    name[0] = '\0';
	  }
	  else {
	    name = vhname;
	  }
#else
	  hp = gethostbyaddr(&cliant.sin_addr,sizeof(struct in_addr),AF_INET);
	  if (hp == 0) 
	      name = unknown;
	  else
	      name = hp->h_name;
#endif
	  printf(" socket #%d   message from %s.\n",msgsock,name);
	  rval = exec_novand(msgsock,argv[0]);
#ifndef VXWORKS
	  close(msgsock);
#endif
      }
      else {
	  if (msgsock == -1 && errno != EINTR) {
	      printf("** ERROR ** accept. errno=%X\n",errno);
	      exit(1);
	  }
      }
  } while(rval==0);

  exit(0);
}

exec_novand(msgsock,proc_name) int msgsock; char *proc_name; {
  char buf[1024];
  int rval, pid, pri, req;
  char *name, *index();
  int i;

  bzero(buf,sizeof(buf));
  rval = read(msgsock,buf,1024);
  if (rval == 0) {
    printf("Ending connection\n");
    return 0;
  }
  if (rval < 0) {
    printf("** ERROR ** reading stream message. errno=%X\n",errno);
    return -1;
  }
  if (rval>100) {
    printf("** ERROR ** too long stream message.\n");
    return -2;
  }
  printf ("Reading %d bytes from socket request,\n    got %s\n",rval,buf);

  name = index(buf,':');
  if ((int)name == 0) return -2;

  name++;
  for(i=0;i<80;i++) {
    if (name[i]==':') break;
    if (name[i]==(char)0) break;
  }
  name[i] = (char)0;

  rval = sscanf(&name[i+1],"%d:%d",&pri,&req);
  if (rval !=2) return -2;

  printf(" name = %s pri=%d req=%d\n",name,pri,req);

  if (strncmp(buf,"tx",strlen("tx"))==0) {
#ifdef VXWORKS
    taskSpawn("novatx",255,0,20000,(FUNCPTR)txproc,msgsock,(int)name,
               pri,req,0,0,0,0,0,0);
#else
    pid = fork();
    if (pid==0) {
      strcpy(proc_name,"novatx");
      exec_tx(msgsock,name,pri,req);
      close(msgsock);
      exit(0);
    }
#endif
  }
  else if (strncmp(buf,"rx",strlen("rx"))==0) {
#ifdef VXWORKS
    taskSpawn("novarx",255,0,20000,(FUNCPTR)rxproc,msgsock,(int)name,
               pri,req,0,0,0,0,0,0);
#else
    pid = fork();
    if (pid==0) {
      strcpy(proc_name,"novarx");
      exec_rx(msgsock,name,pri,req);
      close(msgsock);
      exit(0);
    }
#endif
  }
  else {
    printf("** ERROR ** Invalid message command %s\n",buf);
    return -2;
  }
  return 0;
}

#ifdef VXWORKS
void txproc(msgsock, name, pri, req)
int msgsock, pri, req;
char *name;
{
  printf("Calling txproc %d %s %d %d\n",msgsock,name,pri,req);
  exec_tx(msgsock, name, pri, req);
  close(msgsock);
  exit(0);
}

void rxproc(msgsock, name, pri, req)
int msgsock, pri, req;
char *name;
{
  exec_rx(msgsock, name, pri, req);
  close(msgsock);
  exit(0);
}

#else

#if defined(SGI)
int  child_end() {
#else
void child_end() {
#endif
    while (wait3(0,WNOHANG,0) > 0)
        ;
    return;
}
#endif
