/*
 * bm_com.c
 *
 * Nova buffer manager  program communication part
 *              version 1.01  25-SEP-1942 Masaharu NOMACHI (KEK)
 */

#include <errno.h>
#include "novad.h"
#include "bm.h"
#include "bm_com.h"

struct BMT_process *process_table;
struct BMT_buffer  *buffer_table;
struct msgbuf      *send_msg_buffer;

com_init() {
  send_msg_buffer = (struct msgbuf *)malloc(sizeof(long) + NV_MSGLEN);
  if ((int)send_msg_buffer == -1) {
    perror("buffer manager com malloc");
    return -1;
  }
  return 0;
}

/****************************
 * Initialize the data base *
 ****************************/

init_table(np,nb,size) int np, nb, size; {
  int i, rval, offset;
  
  process_table = (struct BMT_process *)
    malloc(sizeof(struct BMT_process) * np);
  if ((int)process_table == 0) return -1;
  
  buffer_table = (struct BMT_buffer *)
    malloc(sizeof(struct BMT_buffer) * nb);
  if ((int)buffer_table == 0) return -1;

  /*
   * setup process cells
   */
  for(i=0;i<np;i++) {
    process_table[i].p_exec = (char *)0;
    process_table[i].id     = i+BASE_ID;
  }

  /*
   * set up buffer cells
   */
  offset = 0;
  for(i = 0; i<nb; i++) {
    buffer_table[i].id = i+BASE_ID;
    exec_buff_install(&buffer_table[i],offset,size);
    trg_buff_install(&buffer_table[i]);
    offset += size;
  }
  return 0;
}


com_install(ptable,pid,msgid,mtype)
     struct BMT_process *ptable; int pid, mtype; MSG_Q_ID msgid;
{
  struct COM_process *p;
  p = (struct COM_process *)malloc(sizeof(struct COM_process));
  ptable->p_com = (char *)p;
  p->pid   = pid;
  p->table = ptable;
  p->msgid = msgid;
  p->mtype = mtype;
  return;
}

com_remove(ptable) struct BMT_process *ptable; {
  struct COM_process *p;

  p = (struct COM_process *)(ptable->p_com);
  clean_up_msg(p->msgid,p->mtype);
  free((char *)p);
  return;
}


struct BMT_process *assign_ptable(p,pid) struct nova_def *p; int pid; {
  int i;
  for(i=0;i<p->n_process;i++) {
    if (process_table[i].p_exec == (char *)0) return &(process_table[i]);
  }
  return NULL_ptable;
}

struct BMT_process *get_ptable(p,pid,novaid)
     struct nova_def *p; int pid,novaid; 
{
  if (novaid<BASE_ID) return NULL_ptable;
  if (novaid>=(BASE_ID+(p->n_process))) return NULL_ptable;
  return &(process_table[novaid-BASE_ID]);
}

struct BMT_buffer *get_btable(p,offset)
     struct nova_def *p; int offset; 
{
  if (offset<0) return NULL_btable;
  if (offset>=(p->buff_size * p->n_buffer)) return NULL_btable;
  return &(buffer_table[offset/(p->buff_size)]);
}

/******************************************************************/

int init_daemon_msg(p) struct nova_def *p; {
#ifdef VXWORKS
  if (((p->daemon_msgid) = msgQCreate(50,200,MSG_Q_FIFO)) == NULL)   {
#else
  if (((p->daemon_msgid) = msgget(IPC_PRIVATE, IPC_CREAT|0666)) < 0) {
#endif
    perror("novad: msgget");
    return -1;
  }
  p->daemon_type = NV_DAEMON;
  return 0;
}

int init_bufmgr_msg(p) struct nova_def *p; {
#ifdef VXWORKS
  if (((p->bufmgr_msgid) = msgQCreate(50,200,MSG_Q_FIFO)) == NULL)   {
#else
  if (((p->bufmgr_msgid) = msgget(IPC_PRIVATE, IPC_CREAT|0666)) < 0) {
#endif
    perror("novad: msgget");
    return -1;
  }
  p->bufmgr_type  = NV_BM;
  return 0;
}

int set_client_msg(p,novaid,msgid,mtype)
     struct nova_def *p; int novaid, *mtype; MSG_Q_ID *msgid;
{
  if (*msgid == 0) *msgid = p->bufmgr_msgid;
  *mtype = novaid;
  return 0;
}

int receive_bufmgr(p,msg_buffer)
     struct nova_def *p; struct msgbuf *msg_buffer; {
  int rval;

#ifdef VXWORKS
  rval = msgQReceive(p->bufmgr_msgid, msg_buffer->mtext, NV_MSGLEN,
                     WAIT_FOREVER);
#else
  rval = msgrcv(p->bufmgr_msgid, msg_buffer, NV_MSGLEN, p->bufmgr_type, 0);
#endif
  if (rval < 0) {
    perror("buffer manager msgrcv");
    return -1;
  }
#ifdef COMDEBUG
  printf("receive at bufmgr %d %d : %s\n",
	 p->bufmgr_msgid,p->bufmgr_type,msg_buffer->mtext);
#endif
  return 0;
}

int receive_daemon(p,msg_buffer)
     struct nova_def *p; struct msgbuf *msg_buffer; {
  int rval;

#ifdef VXWORKS
  rval = msgQReceive(p->daemon_msgid, msg_buffer->mtext, NV_MSGLEN,
                     WAIT_FOREVER);
#else
  rval = msgrcv(p->daemon_msgid, msg_buffer, NV_MSGLEN, p->daemon_type, 0);
#endif
  if (rval < 0) {
    perror("daemon msgrcv:");
    return -1;
  }
#ifdef COMDEBUG
  printf("receive at daemon %d %d : %s\n",
	 p->daemon_msgid,p->daemon_type,msg_buffer->mtext);
#endif
  return 0;
}

int send_daemon(p,msg_buffer) 
struct nova_def *p; struct msgbuf *msg_buffer; 
{
  int rval;
#ifdef VXWORKS
  rval = msgQSend(p->daemon_msgid, msg_buffer->mtext, NV_MSGLEN, 
                  WAIT_FOREVER, MSG_PRI_NORMAL);
#else
  msg_buffer->mtype = p->daemon_type;
  rval = msgsnd(p->daemon_msgid, msg_buffer, NV_MSGLEN, 0);
#endif
#ifdef COMDEBUG
  printf("send to daemon %d %d: %s\n",
	 p->daemon_msgid,p->daemon_type,msg_buffer->mtext);
#endif
  return rval;
}

int send_bufmgr(p,msg_buffer) 
struct nova_def *p; struct msgbuf *msg_buffer; 
{
  int rval;
#ifdef VXWORKS
  rval = msgQSend(p->bufmgr_msgid, msg_buffer->mtext, NV_MSGLEN, 
                  WAIT_FOREVER, MSG_PRI_NORMAL);
#else
  msg_buffer->mtype = p->bufmgr_type;
  rval = msgsnd(p->bufmgr_msgid, msg_buffer, NV_MSGLEN, 0);
#endif
#ifdef COMDEBUG
  printf("send to bufmgr %d %d : %s\n",
	 p->bufmgr_msgid,p->bufmgr_type,msg_buffer->mtext);
#endif
  return rval;
}


bm_send(ptable,btable,offset,size,significance)
     struct BMT_process *ptable;
     struct BMT_buffer *btable;
     int offset, size, significance;
{
  struct COM_process *p;
  int rval;


  p = (struct COM_process *)(ptable->p_com);
  sprintf(send_msg_buffer->mtext,"ready:%d:%d:%d:%d:",
	  p->pid,offset,size,significance);
  send_msg_buffer->mtype = p->mtype;
#ifdef VXWORKS
  rval = msgQSend(p->msgid, send_msg_buffer->mtext, NV_MSGLEN, 
                  WAIT_FOREVER, MSG_PRI_NORMAL);
#else
  rval = msgsnd(p->msgid, send_msg_buffer, NV_MSGLEN, 0);
#endif
#ifdef COMDEBUG
  printf("send to %d (%d %d) : %s\n",
	 ptable->id,p->msgid,p->mtype,send_msg_buffer->mtext);
#endif
  return rval;
}

int clean_up_msg(id,type) MSG_Q_ID id; int type; {
  int rval;
  char message[NV_MSGLEN+4];

  while(1) {
#ifdef VXWORKS
    rval = msgQReceive(id, message, NV_MSGLEN, NO_WAIT);
    if (rval < 0) return rval;
#else
    rval = msgrcv(id, message, NV_MSGLEN, type, IPC_NOWAIT);
    if (rval < 0) {
      if (errno==ENOMSG) return 0;
      return rval;
    }
#endif
  }
}

