/*
 * novad.c
 *
 * Nova buffer manager daemon program
 *
 *              version 2.00  21-feb-1994 Masa NOMACHI
 *              version 1.07  27-NOV-1993 Yasuo TAKEUCHI     (T.I.T.)
 *              version 1.06  12-APR-1993 Yasuo TAKEUCHI     (T.I.T.)
 *              version 1.02  01-OCT-1992 Masaharu NOMACHI   (KEK online)
 *              version 0.3   22-JUN-1992 Yasuo TAKEUCHI     (T.I.T.)
 *              version 0.1   24-APR-1992 kekonline M. Nomachi
 */

#include "novad.h"

#ifndef VXWORKS
#ifdef SGI
int child_end();
#else
void child_end();
#endif
#endif

#ifdef VXWORKS
#define key_t int
#define SERVER_NUM 1967
void process1(), process2();
#endif

int pid_buffer_manager;
struct nova_def novad;
int sd, msd, pid, novaid, shmid;

#ifdef VXWORKS
novad_main(argv1,argv2,argv3,argv4,argv5,argv6,argv7,argv8,
           argv9,argv10)
char *argv1,*argv2,*argv3,*argv4,*argv5,*argv6,*argv7,*argv8,
           *argv9,*argv10;
{
  char *argv[] = {NULL,argv1,argv2,argv3,argv4,argv5,argv6,argv7,argv8,
           argv9,argv10};
  int argc;
  struct sockaddr_in server, client_addr;
  int client_length;
#else
main(argc, argv) int argc; char **argv; {

  struct sockaddr_un server;
#endif
  int i, rval;
  int key;
  MSG_Q_ID msgid;
#ifndef VXWORKS
  struct sigvec vec;
#endif
#ifdef VXWORKS
  argv[0] = "novad_main";
  client_length = sizeof(client_addr);
  for(i=0;i<11;i++) {
    if(argv[i] == NULL || *argv[i] == '\0') {
      argc = i;
      break;
    }
  }
#endif

#ifndef VXWORKS
  /*
   * Rise process priority if possible
   */
  nice(NV_NICE_VALUE);
#endif

  /*
   * Lock process
   */
/*  plock(PROCLOCK); */

  /*
   * Check arguments
   */
  if (exam_arg(argc, argv, &novad) < 0) exit(1);

  /*
   * Create socket
   */
#ifdef VXWORKS
  bzero(&server, sizeof(server));
  bzero(&client_addr, sizeof(client_addr));
  if ((sd = socket(AF_INET, SOCK_STREAM, 0)) == ERROR)  {
#else
  if ((sd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)       {
#endif
    perror("novad: opening stream socket");
    exit(1);
  }

  /*
   * name socket
   */
#ifdef VXWORKS
  server.sin_family = AF_INET;
  server.sin_port = htons(SERVER_NUM);
  if (bind(sd, (struct sockaddr *)&server, sizeof(server)) == ERROR) {
#else
  server.sun_family = AF_UNIX;
  strcpy(server.sun_path, novad.name);

  /* delete previous socket, if exist */
  /* unlink(novad.name); */

  if (bind(sd, &server, sizeof(struct sockaddr_un))) {
#endif
    perror("novad: binding stream socket");
    exit(1);
  }

#ifndef VXWORKS
  /* change socket permittion (for Ultrix) */
  chmod(novad.name, 0777);
#endif
  /*
   * create "message" and "shared memory"
   */
  if (init_daemon_msg(&novad) < 0) {
    unlink(novad.name);
    exit(1);
  }
  msgid = novad.daemon_msgid;

  if (init_bufmgr_msg(&novad) < 0) {
    unlink(novad.name);
    exit(1);
  }
  if ((shmid = shmget(IPC_PRIVATE, novad.total_size, IPC_CREAT|0666)) < 0) {
#ifdef VXWORKS
    msgQDelete(msgid);
#else
    msgctl(msgid, IPC_RMID, NULL);
#endif
    perror("novad: shmget");
    unlink(novad.name);
    exit(1);
  }

  /* add signal handler */
#ifndef VXWORKS
#if defined(SGI)
  vec.sv_handler = (int (*)())child_end;
#else
  vec.sv_handler = (void (*)())child_end;
#endif
  vec.sv_mask = 0;
  vec.sv_flags = 0;
#endif

#ifdef HP
  sigvector(SIGCHLD, &vec, (struct sigvec *) NULL);
#else
#if !defined(VXWORKS)
  sigvec(SIGCHLD, &vec, (struct sigvec *) NULL);
#endif
#endif

#if !defined(VXWORKS)
  if ((i=fork()) != 0) {
    printf("novad's pid = %d\n",i);
    exit(0);
  }
#endif
  /*
   * start buffer manager 
   */
#ifdef VXWORKS
  pid_buffer_manager = taskSpawn("buf_man", 255, 0, 20000, (FUNCPTR)process1,
                       0,0,0,0,0,0,0,0,0,0);
#else
  if ((pid_buffer_manager = fork()) == 0) {
    /* child process (buffer_manager) */
    close(sd);
    strcpy(argv[0], BUFM_NAME);
    buffer_manager(&novad);
    exit(0);
  }
  else if (pid_buffer_manager == -1)
    perror("novad: fork buffer manager");
#endif

  /*
   * start accept
   */
  listen(sd, novad.n_process);
  while(pid_buffer_manager > 0) {
#ifdef VXWORKS
    if ((msd = accept(sd, (struct sockaddr *)&client_addr, &client_length))
              != ERROR) {
      if (pid_buffer_manager > 0) {
	taskSpawn("server", 255, VX_STDIO, 20000, (FUNCPTR)process2,
		  0,0,0,0,0,0,0,0,0,0);
      }
    }
#else
    if ((msd = accept(sd, 0, 0)) != -1) {
      if (pid_buffer_manager > 0) {
	if(fork() == 0) {
	  /* child process (server) */
	  close(sd);
	  strcpy(argv[0], SERV_NAME);
	  if (exec_install(msd, &novad, shmid, &pid, &novaid) == 0) {
	    nv_server(msd, &novad, pid, novaid);
	  }
	  exit(0);
	}
      }
      close(msd);
    }
#endif
  }
#ifdef DEBUG
  printf("novad: Buffer manager has been killed\n");
#endif
  shmctl(shmid, IPC_RMID, NULL);
#ifdef VXWORKS
  msgQDelete(novad.bufmgr_msgid);
  msgQDelete(novad.daemon_msgid);
#else
  msgctl(novad.bufmgr_msgid, IPC_RMID, NULL);
  msgctl(novad.daemon_msgid, IPC_RMID, NULL);
#endif
  unlink(novad.name);
  close(sd);
}

#ifndef VXWORKS
#ifdef SGI
int  child_end() {
#else
void child_end() {
#endif
  int pid;

  while ((pid = wait3(0,WNOHANG,0)) > 0) {
      if (pid == pid_buffer_manager) pid_buffer_manager = 0;
  }
/*
  pid = wait(0);
  if (pid == pid_buffer_manager) pid_buffer_manager = 0;
*/
#ifdef SGI
  return 0;
#endif
}
#endif

#ifdef VXWORKS
void process1()
{
  buffer_manager(&novad);
  exit(0);
}

void process2()
{
  if (exec_install(msd, &novad, shmid, &pid, &novaid) == 0)  {
    nv_server(msd, &novad, pid, novaid);
  }
  exit(0);
}
#endif
