/*---------------------------------------------------------------*/
/*                                                               */
/*      Cengiz ERBAS                                             */
/*      Superconducting Super Collider Laboratory                */
/*      June 20, 1992                                            */
/*         This is file template.h                               */
/*                                                               */
/*---------------------------------------------------------------*/
#ifdef VXWORKS
#include <msgQLib.h>
#endif
/*---------------------------------------------------------------*/
/*            Return Codes for Different Operations              */
/*---------------------------------------------------------------*/
#define  SUCCESS            0       /* Operation successful      */
#ifndef ERROR
#define  ERROR             -1       /* Operation failed          */ 
#endif
#ifndef TRUE
#define  TRUE               1       /* Logical one               */ 
#endif
#ifndef FALSE
#define  FALSE              0       /* Logical zero              */ 
#endif
/*---------------------------------------------------------------*/
/*             Definitions for the Command Table                 */
/*---------------------------------------------------------------*/
#define  TABLE_KEY          185     /* Common Table access key   */
#define  HEADER_KEY         186     /* Common Header access key  */
#define  TABLE_FLAG         0666    /* Common access flag        */
#define  TABLE_SIZE         25      /* Max number of commands    */

/*---------------------------------------------------------------*/
/*               Definitions for the Commands                    */
/*---------------------------------------------------------------*/
#define  CMD_LENGTH         400     /* Command Length            */ 
#define  CMD_NAME           10      /* Command Name Length       */
#define  PROCESS_NAME       20      /* Process Name Length       */
#define  NODE_NAME          60      /* Node name length          */

#define  NOT_CMD            -2
#define  AMBIGUOUS_CMD      -1

/*---------------------------------------------------------------*/
/*             Buffer (Record) Type Definitions                  */
/*---------------------------------------------------------------*/
#define  EVENT_RECORD       0       /* Event Data Buffer         */ 
#define  BEGINRUN_RECORD    1       /* Begin Run Buffer          */ 
#define  PAUSERUN_RECORD    2       /* Pause Run Buffer          */ 
#define  RESUMERUN_RECORD   3       /* Resume Run Buffer         */ 
#define  ENDRUN_RECORD      4       /* End Run Buffer            */ 
#define  CALIBRATION_RECORD 5       /* Calibration Buffer        */ 
#define  ENVIRONMENT_RECORD 6       /* Environment Buffer        */ 

/*---------------------------------------------------------------*/
/*                   Other Definitions                           */
/*---------------------------------------------------------------*/
#define  MAX_MSG_SIZE       256     /* Maximum message size      */
#define  MSG_TYPE           1       /* Maximum message size      */

#define  ALIVE              1       /* The process is alive      */
#define  DEAD               2       /* The process is dead       */
#define  BUSY               3       /* The process is busy       */

#define  VAR_NAME           20      /* Variable Name Length      */

#define  INTEGER            1       /* int for var_dict          */
#define  STRING             2       /* (char*) for var_dict      */
#define  REAL               3       /* float for var_dict        */

#define  EVENT_SIGNAL       SIGUSR1 /* Signal type for events    */
#define  MSG_SIGNAL         SIGUSR2 /* Signal type for messages  */
#define  BUSY_SIGNAL        SIGQUIT /* Signal type to test process status */

/*---------------------------------------------------------------*/
/*                  Structure Definitions                        */
/*---------------------------------------------------------------*/
typedef struct header Header;
typedef struct table  Table;

struct header {
   int first;
   int last;
};

struct table {
   char process_name[PROCESS_NAME];
   char node[NODE_NAME];
   int  process_id;
#ifdef VXWORKS
   MSG_Q_ID msgq_id;
#else
   int  msgq_id;
#endif
   int  status;
};

struct command_list {
   char string[CMD_LENGTH];
   struct command_list *next;
};

struct msgbuf1 {
   long mtype;
   char mtext[8192];
};


#ifndef MY_MSGBUF
#define MY_MSGBUF
struct my_msgbuf
{
	long int mtype;
	char mtext[512];
};
#define msgbuf my_msgbuf
#endif


struct data_record {
   int rec_word_count;
   int buffer_type;
   int run_number;
   int evnt_number;
   int rec_mode;
   int unused1;
   union {
      struct {
         int data[100];
      } event_record;
      struct {
	 char time[28];
	 char comment[100];
      } beginrun_record;
      struct {
	 char time[28];
	 char comment[100];
      } pauserun_record;
      struct {
	 char time[28];
	 char comment[100];
      } resumerun_record;
      struct {
	 char time[28];
	 char comment[100];
      } endrun_record;
      struct {
	 char time[28];
	 char comment[100];
      } calibration_record;
      struct {
	 char time[28];
	 char comment[100];
      } environment_record;
   } type_of_record;
};

