/*************************************************
 * CVS Log
 * mur_c_user.c,v
 * Revision 1.1.1.1  1993/07/14  19:43:22  dbrenner
 * UNIDAQ 2.1 distribution - July 14,1993
 *
*************************************************/
/* ==========================================================================
**
** MUR_C_USER.C
**
** DESCRIPTION
**	This file contains all the "C" routines which are available to the
**	programmer for reporting messages via the MURMUR message reporting
**	system.  These routines make up the highest level of the MURMUR 
**	client library.
**
** DEVELOPERS
**	Gary P. Sergey
**
**	Fermilab Data Acquisition Group
**	Batavia, Il 60510, U.S.A.
**
** MODIFICATIONS
**         Date       Initials	Description
**	18-Feb-1992	GPS	Original Entry
** 
** ENTRY POINTS			 SCOPE
**	mur_send		external
**	mur_set_proc_name	external
**	mur_send_text		external
**	mur_open		external
**	mur_server_add		external (VxWorks only)
**	mur_route_send		external (VxWorks only)
**	mur_route_start		external (VxWorks only)
**	mur_route_send_text	external (VxWorks only)
*/

#include "mur_cond.h"

#ifdef MUR_VMS
#include <stdarg.h>
#include "MULTINET_COMMON_ROOT:[multinet.include.rpc]rpc.h"
#include "MULTINET_COMMON_ROOT:[multinet.include.rpc]xdr.h"
#endif

#ifdef MUR_VXWORKS
#include <vxWorks.h>
#include <msgQLib.h>
#ifdef MUR_STDC
#include <stdarg.h>
#else
#include <varargs.h>
#endif 
#include <rpc/rpctypes.h>
#include <rpc/xdr.h>
#endif

#ifdef MUR_UNIX
#ifdef MUR_STDC
#include <stdarg.h>
#else
#include <varargs.h>
#endif 
#include <stdio.h>	/* Need for SGI (__file_s declaration) */
#include <rpc/types.h>
#include <rpc/xdr.h>
#endif /* MUR_UNIX */

#include "mur_def.h"
#include "mur_func.h"
#include "mur_errtab.h"
#include "errno_msg_c.h"
#include "mur_macros.h"
#include "murmur_msg_c.h"

/* ==========================================================================
**
** GLOBAL CLIENT CONTEXT STRUCTURE DEFINITION AND INITIALIZATION
*/
#ifdef MUR_VXWORKS
MUR_CLIENT_CTX *global_cctx_ptr;
#else
MUR_CLIENT_CTX global_cctx = MUR_CTX_INIT;
#endif

#ifdef MUR_VXWORKS
/* ==========================================================================
**
** VXWORKS GLOBAL SERVER ADDRESS STRING
*/
MUR_SADD murmur_server_add;

/* ==========================================================================
**
** VXWORKS GLOBAL ISP MESSAGE QUEUE ID
*/
MSG_Q_ID global_msgq_id;
#endif


/* ==========================================================================
**
** ROUTINE: mur_send
*/
#if defined(MUR_STDC) || defined (MUR_VMS)
unsigned long mur_send(unsigned long a_msgcode, int a_numargs, ...)
#else
unsigned long mur_send(va_alist)

va_dcl
#endif
/*
** DESCRIPTION
** 	MUR_SEND will send a message code and optionally a variable number
** 	of message arguments (up to 8 maximum) and/or optionally one 
** 	additional (appended) text string to the MURMUR server.  This routine
**	can also be used to send a single "text only" message.
** 
** ARGUMENTS
**      a_msgcode
**	    Type:        unsigned long
**	    Access:      read only
**	    Mechanism:   by value
**	    Description: The message to be reported.
**
**      a_numargs
**	    Type:	 int
**	    Access:	 read only
**	    Mechanism:	 by value
**	    Description: The number of arguments to be passed (up to 8 message
**	    arguments and one appended text argument), where each argument
**	    actually consists of a pair of variables (the argument itself,
**	    followed by the argument type).  It is up to the caller to ensure
**	    the proper order, type, and number of arguments is passed.
**
**	    Note that (of the 8 message arguments) only 4 string arguments are
**	    allowed, and are limited to 30 characters each.  An appended text
**	    string can be up to 132 characters.
**
**	    If the argument is "text only", no other arguments are allowed.
**
**	    A value of 0 indicates no message arguments or appended text are
**	    being supplied.
**
**      a_argtype
**	    Type:	 MUR_ARG_TYPE
**	    Access:	 read only
**	    Mechanism:	 by value
**	    Description: The variable type of the subsequent argument (must be
**	    one of the defined MUR_ARG_TYPE values).  This argument is optional.
**
**      a_arg
**	    Type:	 as declared by the preceding a_argtype
**	    Access:	 read only
**	    Mechanism:	 by reference
**	    Description: The actual message argument or the appended text 
**	    string.  This argument is optional.
**
** VALUES RETURNED
**	MUR_SUCCESS       - Message sent to MURMUR server successfully.
**	MUR_LCL_SWITCH    - Unable to communicate with server, switched 
**			    to local mode (messages go to std error) 
**	MUR_SEV_ERR       - Unable to send message to MURMUR server.
**	
**
** CALLED BY
**      user's application
**
** CALLS TO
**	mur_com_cleanup
**	mur_com_send
**	mur_get_ctx
**      mur_internal_err
**	mur_print_msg
**	mur_rec_hdr_pack
**	mur_rec_pack
**	mur_time_date
**	various XDR routines
**
** ========================================================================== 
*/
{ /* begin: mur_send */

#if defined(MUR_STDC) || defined(MUR_VMS)
va_list		ap;
#else
va_list		ap;
unsigned long	a_msgcode;
int		a_numargs;
#endif

MUR_CLIENT_ERR	err_status;	    /* MURMUR client internal error status */
int		argcntr	    = 0;
int	        msgargcount = 0;
int		appendcount = 0;
int	        stringcount = 0;
MUR_BOOL	prnt_errno  = FALSE;
MUR_ARG_FLAGS	argtypeflags;
MUR_EXTRA_FLAGS extraflags;
char		*argtypeflagsptr, *extraflagsptr;
unsigned int    argtypelen, extralen;
MUR_ARG_TYPE	argtype;
long		arglong;
unsigned long	argulong;
float		argfloat;
XDR		*xdrs;
int		saved_xdr_position;
MUR_CLIENT_CTX  *ctxptr;
char		*lstrptr;
MUR_CTIME	ltime;
MUR_CDATE	ldate;
MUR_MSG_TYPE_ENUM msgtype;
MUR_ARGSTRING_PTR argstring;
MUR_APPTEXT_PTR	  appstring;
MUR_TEXTONLY_PTR  textstring;
/*
** GET POINTERS TO FLAG VARIABLES 
*/
argtypeflagsptr = (char *)&argtypeflags.c[0];
extraflagsptr   = (char *)&extraflags.c[0];

/*
** FOR NON-ANSI "C", GET THE FIXED ARGUMENTS (MESSAGE CODE AND NUMBER OF
** ARGUMENTS)
*/
#if !defined(MUR_STDC) && !defined(MUR_VMS)
   va_start(ap);
   a_msgcode = va_arg(ap, unsigned long);
   a_numargs = va_arg(ap, int);
#endif

/*---------------------------------------------------------------------------
**
** RETRIEVE CONTEXT FOR THIS CLIENT - OR CREATE CONTEXT IF NEW CLIENT
*/
err_status = mur_get_ctx(&ctxptr, a_msgcode);
if (err_status != MUR_SUCCESS) goto mur_ctx_err_exit;

/*---------------------------------------------------------------------------
**
** PACK MESSAGE INFORMATION FIELDS INTO USER RECORD
*/

/* Get the client time and date in ASCII */
mur_time_date((MUR_CTIME_PTR)ltime, (MUR_CDATE_PTR)ldate);

/* Set XDR structure to point to user record area */
xdrmem_create((XDR *) &ctxptr->xdrs, (char *)ctxptr->user_rec,
              MUR_USER_REC_LEN, XDR_ENCODE);

xdrs = (XDR *) &ctxptr->xdrs;  /* get XDR handle pointer */

/* Pack message type */
msgtype = MSG_NORMAL;
if (!(xdr_enum(xdrs, (enum_t *)&msgtype)))
   {err_status = MUR_CERR_REC_PACK; goto mur_err_exit;}

err_status = mur_rec_pack(xdrs, &a_msgcode, (MUR_CTIME_PTR)ltime, 
                          (MUR_CDATE_PTR)ldate);
if (err_status != MUR_SUCCESS)
   {
   err_status = MUR_CERR_REC_PACK;
   goto mur_err_exit;
   }


/*---------------------------------------------------------------------------
**
** PACK ARGUMENT FLAGS AND ANY USER ARGUMENTS INTO USER RECORD TOO 
*/

/*
** ADJUST THE XDR STRUCTURE TO POINT PAST THE ARGUMENT FLAGS 
*/
/* 
   The flags identifying what arguments were passed by the user must be in
   the user record first so the server can determine what to unpack.
   However, since we do not know what the user has passed yet, pack dummy
   arguments for now (to adjust the XDR pointer), then pack the correct 
   flags later.

   F.Y.I., the "argtypeflags" union is used to keep track of the order,
   number, and type of message arguments.  This union is passed along to
   the server, which will compare the words with the words defined in the
   database for the message.  If the words do not match, the arguments were
   passed incorrectly.  The "extraflags" union is used similarly to keep
   track of of whether or not an appended text srting and/or a process name
   prefix are present.
*/
argtypeflags.l[0] = 0;
argtypeflags.l[1] = 0;
extraflags.l      = 0;
argtypelen = MUR_NUM_ARGS;
extralen   = sizeof(long);

saved_xdr_position = (int) xdr_getpos(xdrs); /* Save buffer pointer for later */
if (!(xdr_bytes(xdrs, (char **)&argtypeflagsptr, &argtypelen, argtypelen)))
   {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}
if (!(xdr_bytes(xdrs, (char **)&extraflagsptr, &extralen, extralen)))
    {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}

/*
** IF THERE ARE ARGUMENTS, LOOP FOR EACH PASSED ARGUMENT PAIR 
*/

if (a_numargs > 0)
   {
#if defined(MUR_STDC) || defined(MUR_VMS)	    /* ANSI C */
   va_start(ap, a_numargs);
#endif
   while(argcntr++ < a_numargs)
      {

      /*
      ** GET ARGUMENT TYPE FIRST
      */
      argtype = (MUR_ARG_TYPE)va_arg(ap, MUR_ARG_TYPE);

      /* 
      ** NOW GET THE ACTUAL ARGUMENT AND PACK IT INTO THE USER RECORD
      */
      switch (argtype)
         {
         case MUR_LONG:
            /* Update type-of-argument string */
            argtypeflags.c[msgargcount] = (MUR_ARG_TYPE_BYTE) MUR_LONG;
            /* Update # of arguments so far */
            msgargcount++;
            /* Get long integer argument */
            arglong = (long)va_arg(ap, long);
            /* Try packing that baby */
            if (!(xdr_long(xdrs, &arglong)))
               {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}
            break;

         case MUR_U_LONG:
            /* Update type-of-argument string */
            argtypeflags.c[msgargcount] = (MUR_ARG_TYPE_BYTE) MUR_U_LONG;
            /* Update # of arguments so far */
            msgargcount++;                   
            /* Get unsigned long integer argument */
            argulong = (u_long)va_arg(ap, u_long); 
            /* Try packing this sucker */
            if (!(xdr_u_long(xdrs, &argulong)))
               {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}
            break;

         case MUR_FLOAT:
            /* Update type-of-argument string */
            argtypeflags.c[msgargcount] = (MUR_ARG_TYPE_BYTE) MUR_FLOAT;
            /* Update # of arguments so far */
            msgargcount++;                   
            /* Get single precision floating point argument */
            argfloat = (float) va_arg(ap, float);
            /* Pack that mother */
            if (!(xdr_float(xdrs, &argfloat)))
               {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}
            break;

         case MUR_STRING:
            /* Update type-of-argument string */
            argtypeflags.c[msgargcount] = (MUR_ARG_TYPE_BYTE) MUR_STRING;
            /* Update # of arguments so far */
            msgargcount++;
            /* Make sure too many strings were not passed */
            if (++stringcount > MUR_NUM_STR_ARGS)
               {err_status = MUR_CERR_STR_ARGS; goto mur_err_exit;}
            /* Get string argument */
            argstring = (MUR_ARGSTRING_PTR)va_arg(ap, MUR_ARGSTRING_PTR);
            /* Pack til ya puke */
            if (!(xdr_string(xdrs, (char **) &argstring, MUR_ARG_STRING_LEN)))
               {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}
            break;

         case MUR_APPEND:
            /* Make sure only one appended text string was passed */
            if (++appendcount > MUR_NUM_APPEND)
               {err_status = MUR_CERR_APP_ARGS; goto mur_err_exit;}
            /* Get appended text argument, but DON'T pack it yet */
            appstring = (MUR_APPTEXT_PTR)va_arg(ap, MUR_APPTEXT_PTR);  
            break;

         case MUR_TEXT_ONLY_MSG:
            /* Make sure this is the only argument passed */
            if (a_numargs > 1)
               {err_status = MUR_CERR_TEXT_ONLY; goto mur_err_exit;}
            /* Update type-of-argument string */
            argtypeflags.c[msgargcount] = (MUR_ARG_TYPE_BYTE) MUR_TEXT_ONLY_MSG;
            msgargcount++;
            /* Get "text only" argument */
            textstring = (MUR_TEXTONLY_PTR)va_arg(ap, MUR_TEXTONLY_PTR);  
            /* Pack it, or get out */
            if (!(xdr_string(xdrs, (char **) &textstring, MUR_TEXT_ONLY_LEN)))
               {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}
            break;

         default:
            err_status = MUR_CERR_INV_ARG; 
            goto mur_err_exit;
            break;
         } /* switch */

      /*
      ** MAKE SURE THAT TOO MANY ARGUMENTS WERE NOT PASSED
      */

      if (msgargcount > MUR_NUM_ARGS)
         {err_status = MUR_CERR_ARGS; goto mur_err_exit;}

      } /* while */

#if defined(MUR_STDC) || defined(MUR_VMS)	    /* ANSI C */
   va_end(ap);  /* Variable argument list end */
#endif

   } /* if (a_numargs > 0) ... */

#if !defined(MUR_STDC) && !defined(MUR_VMS)	    /* non-ANSI C */
va_end(ap);  /* Variable argument list end */
#endif

/*
** ALWAYS PACK THE APPENDED TEXT STRING AND/OR THE PROCESS NAME PREFIX LAST 
** (IF PRESENT)
*/

if (appendcount)		    /* Appended text string */
   {
   /* Update extra flags union */
   extraflags.c[MUR_APP_INDEX] = (MUR_BOOL) TRUE;
   if (!(xdr_string(xdrs, (char **) &appstring, MUR_APP_TEXT_LEN)))
         {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}
   }

if (ctxptr->new_prefix == PFX_SEND) /* Process name prefix */
   {
   /* Update extra flags union */
   extraflags.c[MUR_PREFIX_INDEX] = (MUR_BOOL) TRUE;
   /* Indicate that prefix has been sent (if all goes well) */
   ctxptr->new_prefix = PFX_DEFINED;
   lstrptr = (char *)ctxptr->prefix; 
   if (!(xdr_string(xdrs, (char **) &lstrptr, MUR_PREFIX_LEN)))
         {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}
   }

/*
** NOW GO BACK AND PACK THE CORRECT ARGUMENT FLAGS
*/

/* Get old buffer position */
if (!(xdr_setpos(xdrs, (u_int)saved_xdr_position))) 
    {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}

/* Pack the flags */
argtypelen = MUR_NUM_ARGS;
extralen   = sizeof(long);
if (!(xdr_bytes(xdrs, (char **)&argtypeflagsptr, &argtypelen, argtypelen)))
   {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}
if (!(xdr_bytes(xdrs, (char **)&extraflagsptr, &extralen, extralen)))
    {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}

/*---------------------------------------------------------------------------
**
** INITIALIZE COMMUNICATION LINK WITH SERVER IF NECESSARY 
*/
if (ctxptr->cstate == CTX_NEW_LINK)   /* Initialize if this is a new client */
   {
   err_status = mur_client_init(ctxptr, a_msgcode);
   if (err_status != MUR_SUCCESS) 
      {
      prnt_errno  = TRUE; 
      if (ctxptr->cstate == CTX_LOCAL)
         {
         mur_print_msg((char *)ctxptr->user_rec);   /* Print msg to stderr */
         }
      goto mur_err_exit;
      }
   }

/*---------------------------------------------------------------------------
**
** SEND USER RECORD TO MURMUR SERVER 
*/

/* If local mode then send message to stderr only... */
if (ctxptr->cstate == CTX_LOCAL)
   {
   mur_print_msg((char *)ctxptr->user_rec);   /* Print msg to stderr */
   }

/* ...otherwise, send user record to server */
else
   {
   err_status = mur_com_send(ctxptr, (char *)ctxptr->user_rec, MUR_SEND_SIZE);
   if (err_status != MUR_SUCCESS) goto mur_com_err_exit;
   }

/*---------------------------------------------------------------------------
**
** RETURN FROM MUR_SEND
*/

/*
** NORMAL SUCCESSFUL COMPLETION 
*/
return(MUR_SUCCESS);

/*
** COMMUNICATION ERROR 
*/
mur_com_err_exit:
mur_com_cleanup(ctxptr);    /* Close socket and clean up client context */
prnt_errno  = TRUE;	    /* Print errno when mur_internal_err is called */

/*
** ERROR EXIT 
*/
mur_err_exit:

/* If the prefix has already been sent to the server, reset the state so
   the next time the user calls a MURMUR routine, the prefix is re-sent to
   the server */
if (ctxptr->new_prefix == PFX_DEFINED)
   {ctxptr->new_prefix = PFX_SEND;}

mur_ctx_err_exit:
/* Report error as best we know how */
mur_internal_err(err_status, a_msgcode, prnt_errno); 
if(err_status > MUR_MAX_COM_ERROR)
  return(MUR_SEV_ERR);	/* packing or args */
else
  return(MUR_LCL_SWITCH);	/* com failure, switched to local mode */

} /* end: mur_send */



/* ==========================================================================
**
** ROUTINE: mur_set_proc_name
*/
#ifdef MUR_STDC
unsigned long mur_set_proc_name(char *a_prefix)
#else
unsigned long mur_set_proc_name(a_prefix)
char 	*a_prefix;
#endif
/*
** DESCRIPTION
**	Sets the process name prefix to the indicated text string.  All
**	subsequent MURMUR messages reported by the current process will 
**	then be prefixed with this string.
** 
** ARGUMENTS
**      a_prefix
**	    Type:	 char[MUR_PREFIX_LEN]
**	    Access:	 read only
**	    Mechanism:	 by reference
**	    Description: The process name prefix character string.
**
** VALUES RETURNED
**	MUR_SUCCESS   - Process name prefix set successfully.
**	MUR_SEV_ERR   - Routine failed miserably.
**
** CALLED BY
**      user's application
**
** CALLS TO
**	mur_get_ctx
**      mur_internal_err
**
** ========================================================================== 
*/
{ /* begin: mur_set_proc_name */

MUR_CLIENT_ERR	err_status;	    /* MURMUR client internal error status */
MUR_CLIENT_CTX  *ctxptr;

/*---------------------------------------------------------------------------
**
** RETRIEVE CONTEXT FOR THIS CLIENT - OR CREATE CONTEXT IF NEW CLIENT
*/
err_status = mur_get_ctx(&ctxptr, 0);
if (err_status != MUR_SUCCESS) goto mur_err_exit;

/*---------------------------------------------------------------------------
**
** SET THE PROCESS NAME PREFIX 
*/
/* Save copy of passed prefix in client context */
strncpy(ctxptr->prefix, a_prefix, MUR_PREFIX_LEN);
ctxptr->prefix[MUR_PREFIX_LEN] = (char)0;  /* Null terminate, just in case */

/* Set flag so that prefix is included in next message to server */
ctxptr->new_prefix = PFX_SEND;

/*---------------------------------------------------------------------------
**
** RETURN FROM MUR_SET_PROC_NAME
*/

/*
** NORMAL SUCCESSFUL COMPLETION 
*/

return(MUR_SUCCESS);

/*
** ERROR EXIT 
*/

mur_err_exit:
/* Report error as best we know how */
mur_internal_err(err_status, 0, (MUR_BOOL)FALSE);
return(MUR_SEV_ERR);

} /* end: mur_set_proc_name */


/* ==========================================================================
**
** ROUTINE: mur_open
*/
unsigned long mur_open()
/*
** DESCRIPTION
**	MUR_OPEN is an optionally called routine, used to open a communication
**	link between the current application (client) and the Murmur server.
**	This routine is not required to be called, since by default, the link
**	is automatically established when the application makes its first call
**	to a Murmur message routine.
**
**	Some programmers may not want to incur the automatic overhead of
**	establishing a communication link during time critical portions of
**	their application.  So MUR_OPEN allows the link to be established on
**	demand, before entering the critical code.  MUR_OPEN is also useful
**	for checking if the Murmur server is present before continuing.
** 
** ARGUMENTS
**      N/A
**
** VALUES RETURNED
**	MUR_SUCCESS    - Connection to server opened successfully.
**	MUR_LCL_SWITCH - Unable to communicate with server, switched 
**			    to local mode (messages go to std error) 
**	MUR_SEV_ERR    - Routine failed miserably.
**
** CALLED BY
**      user's application
**
** CALLS TO
**	mur_get_ctx
**      mur_internal_err
**
** ========================================================================== 
*/
{ /* begin: mur_open */

MUR_CLIENT_ERR	err_status;	    /* MURMUR client internal error status */
MUR_CLIENT_CTX  *ctxptr;

/*
** RETRIEVE CONTEXT FOR THIS CLIENT - OR CREATE CONTEXT IF NEW CLIENT
*/
err_status = mur_get_ctx(&ctxptr, 0);
if (err_status != MUR_SUCCESS) goto mur_err_exit;

/*
** INITIALIZE INTERNAL CLIENT STRUCTURES AND OPEN A COMMUNICATION LINK WITH
** THE SERVER
*/
err_status = mur_client_init(ctxptr, 0);
if (err_status != MUR_SUCCESS) goto mur_err_exit;

/*
** NORMAL SUCCESSFUL COMPLETION 
*/
return(MUR_SUCCESS);

/*
** ERROR EXIT 
*/
mur_err_exit:
/* Report error as best we know how */
mur_internal_err(err_status, 0, (MUR_BOOL)FALSE);
if(err_status > MUR_MAX_COM_ERROR)
   return(MUR_SEV_ERR);
else
   return(MUR_LCL_SWITCH);

} /* end: mur_open */


/* ==========================================================================
**
** ROUTINE: mur_translate_vms
*/
#ifdef MUR_STDC
unsigned long mur_translate_vms(unsigned long vms_code, unsigned long *mur_code)
#else
unsigned long mur_translate_vms(vms_code, mur_code)
unsigned long vms_code;
unsigned long *mur_code;
#endif
/*
** DESCRIPTION
** 
**	Translates a VMS formatted message code into it's MURMUR 
**	equivalent for error reporting through MURMUR
** 
** ARGUMENTS
**	vms_code
**	    Type: 	 unsigned long
**	    Access:	 read only
**	    Mechanism:	 by value
**	    Description: The VMS formatted message code
**
**	mur_code
**	    Type: 	 unsigned long
**	    Access:	 write only
**	    Mechanism:	 by reference
**	    Description: translated version of VMS message code returned to 
**			 user
**
** VALUES RETURNED
**	mur_code      - murmur msg code translated from VMS message code
**	MUR_SUCCESS   - Routine successfully translated message
**	MUR_CANT_TRANSLATE - Routine failed to translate vms code
**
** CALLED BY
**      user's application
**
** CALLS TO
**
** ========================================================================== 
*/
{ /* begin: mur_translate */

MUR_CLIENT_ERR err_status;
long int mur_msg=0;
long int mur_severity=0;
long int mur_facility=0;

/*
** Verify whether or not this is a VMS system message i.e. vms message
** code only will be in vms_code, no facility and no severity.
*/

if ( (vms_code & VMS_M_FAC) == 0 )
   {
   mur_msg = vms_code;
   if (vms_code == 1)
      mur_severity = MUR_SEV_SUCCESS;
   else
      mur_severity = MUR_SEV_ERROR;
   goto get_facility;
   }

/* 
** VMS message field is 13 bits and murmur is 11 bits. Verify
** vms msg code can be translated
*/

mur_msg = (vms_code & VMS_M_MSG) >> VMS_LSB_MSG;
if ( mur_msg > MUR_MAX_MSG_NUMBER) goto mur_err_exit;

mur_severity = vms_code & VMS_M_SEV;
if (mur_severity == VMS_SEV_WARNING)
   mur_severity = MUR_SEV_WARNING;
else if ((mur_severity == VMS_SEV_ERROR) || (mur_severity == VMS_SEV_FATAL))
   mur_severity = MUR_SEV_ERROR;
else if ((mur_severity == VMS_SEV_SUCCESS) || (mur_severity == VMS_SEV_INFORM))
   {
   }
else
   goto mur_err_exit;

get_facility:

mur_facility = ( (vms_code & VMS_M_FAC) >> VMS_LSB_FAC) + MUR_VMS_FAC_OFFSET;

*mur_code = ((unsigned long)(1) << MUR_MSB)
	 | (mur_facility << MUR_LSB_FAC)
	 | (mur_msg << MUR_LSB_MSG)
	 | mur_severity;

/*---------------------------------------------------------------------------
**
** RETURN FROM MUR_TRANSLATE_VMS
*/

/*
** NORMAL SUCCESSFUL COMPLETION 
*/

return(MUR_SUCCESS);

/*
** ERROR EXIT 
*/

mur_err_exit:
/* Report error as best we know how */
err_status = MUR_CERR_CANT_TRANSLATE;
mur_internal_err(err_status, 0, (MUR_BOOL)FALSE);
return(MUR_SEV_ERR);

} /* end: mur_translate_vms */


/* ==========================================================================
**
** ROUTINE: mur_translate_unix
*/
#ifdef MUR_STDC
unsigned long mur_translate_unix(int errno_code, unsigned long *mur_code)
#else
unsigned long mur_translate_unix(errno_code, mur_code)
int  errno_code;
unsigned long *mur_code;
#endif
/*
** DESCRIPTION
** 
**	Translates a VMS formatted message code into it's MURMUR 
**	equivalent for error reporting through MURMUR
** 
** ARGUMENTS
**	errno_code
**	    Type: 	 int
**	    Access:	 read only
**	    Mechanism:	 by value
**	    Description: unix errno error code
**
**	mur_code
**	    Type: 	 unsigned long
**	    Access:	 write only
**	    Mechanism:	 by reference
**	    Description: translated version of VMS message code returned to 
**			 user
**
** VALUES RETURNED
**	mur_code      - murmur msg code translated from VMS message code
**	MUR_SUCCESS   - Routine successfully translated message
**	MUR_CANT_TRANSLATE - Routine failed to translate vms code
**
** CALLED BY
**      user's application
**
** CALLS TO
**
** ========================================================================== 
*/
{ /* begin: mur_translate_unix */

MUR_CLIENT_ERR err_status;

   if ( (errno_code >= MUR_ERRTABSIZE) || (errno_code <=0) )
      *mur_code = errno_UNKNOWN;
   else
      *mur_code = mur_errnoTbl[errno_code];

/*---------------------------------------------------------------------------
**
** RETURN FROM MUR_TRANSLATE_UNIX
*/

/*
** NORMAL SUCCESSFUL COMPLETION 
*/

return(MUR_SUCCESS);

/*
** ERROR EXIT 
*/

mur_err_exit:
/* Report error as best we know how */
err_status = MUR_CERR_CANT_TRANSLATE;
mur_internal_err(err_status, 0, (MUR_BOOL)FALSE);
return(MUR_SEV_ERR);

} /* end: mur_translate_unix */


/* ==========================================================================
**
** DESCRIPTION
** 	MUR_SEND_TEXT is a function which invokes MUR_SEND to send a
**      message containing text only to the MURMUR server.
** 
** ARGUMENTS
**      a_text
**	    Type:        char[132]
**	    Access:      read only
**	    Mechanism:   by reference
**	    Description: The text string message to be reported.
**
** VALUES RETURNED
**	MUR_SUCCESS      - Message sent to MURMUR server successfully.
**	MUR_SEV_ERR      - Unable to send message to MURMUR server.
**
** CALLED BY
**      user's application
**
** ========================================================================== 
*/

#ifdef MUR_STDC
unsigned long mur_send_text(char *a_text)
#else
unsigned long mur_send_text(a_text)
char *a_text;
#endif
        {
        return (mur_send(0, 1, MUR_TEXT_ONLY_MSG, a_text));
        }
 


#ifdef MUR_VXWORKS
/* ==========================================================================
**
** ROUTINE: mur_server_add
*/
#ifdef MUR_STDC
void mur_server_add
(
char		*a_sadd		   /* IN:  Server address in ASCII */
)
#else
void mur_server_add(a_sadd)

char		*a_sadd;		   /* IN:  Server address in ASCII */
#endif
/*
** DESCRIPTION
** 	Used to set the MURMUR server Internet address under VxWorks.  This
**	routine must be called before any client (user) applications attempt
**	to send messages to the MURMUR server.  If this routine is not called
**	beforehand, the message codes will be sent to stderr instead.
** 
** VALUES RETURNED
**	N/A
** 
** CALLED BY
**      User (via VxWorks shell)
**
** GLOBALS REFERENCED
**	murmur_server_add
**
** ========================================================================== 
*/
{ /* begin: mur_server_add */

strncpy(murmur_server_add, a_sadd, MUR_SADD_LEN);
murmur_server_add[MUR_SADD_LEN] = (char)0;

} /* end: mur_server_add */


/* ==========================================================================
**
** ROUTINE: mur_route_send
*/
void mur_route_send(va_alist)

va_dcl
/*
** DESCRIPTION
**	MUR_ROUTE_SEND provides functionality similar to mur_send for use in
**	VxWorks interrupt service procedures (ISP's).  See MUR_SEND description
**	for more details.  Since ISP's cannot call functions which block, 
**	MUR_SEND (which uses blocking I/O) cannot be called directly.
**
**	MUR_ROUTE_SEND does not call the MURMUR server directly, but instead
**	contacts another task to send the message.  MUR_ROUTE_SEND simply
**	forwards the passed parameters down a message queue to the MURMUR
**	VxWorks router process, which then uses mur_send to make the actual
**	call to the MURMUR server.  The router process must be present or the
**	message will be lost.
**
**	NOTE: If an improper number, sequence, or type of arguments is passed,
**	      only the message code will be sent.
** 
** ARGUMENTS
**	Same as mur_send
**
** VALUES RETURNED
**	N/A
**
** CALLED BY
**      user's application
**
** CALLS TO
**	bcopy
**	msgQSend
**
** GLOBALS REFERENCED
**	global_msgq_id
**
** ========================================================================== 
*/
{ /* begin: mur_route_send */

va_list		ap;
unsigned long	a_msgcode;
int		a_numargs;
int		argcntr	    = 0;
int	        msgargcount = 0;
int		appendcount = 0;
int	        stringcount = 0;
MUR_BOOL	get_out = FALSE;
MUR_ARG_TYPE	argtype;
long		arglong;
unsigned long	argulong;
float		argfloat;
MUR_ARGSTRING_PTR argstring;
MUR_APPTEXT_PTR	  appstring;
MUR_TEXTONLY_PTR  textstring;
char		l_buf[MUR_USER_REC_LEN];
char		*l_bufptr;
char		*l_numargptr;
u_long		l_bytes;

va_start(ap);

/*
** GET MESSAGE CODE AND NUMBER OF ARGUMENTS AND SAVE IN BUFFER
*/
a_msgcode = va_arg(ap, unsigned long);
a_numargs = va_arg(ap, int);
l_bufptr = l_buf;        /* Set buffer ptr to start of buffer */
bcopy((char *)&a_msgcode, l_bufptr, sizeof(unsigned long));
l_bufptr += sizeof(unsigned long);

l_numargptr = l_bufptr;  /* Save "number of argument" ptr in case of error */
bcopy((char *)&a_numargs, l_bufptr, sizeof(int));
l_bufptr += sizeof(int);

/*
** SAVE ANY MESSAGE ARGUMENTS IN THE BUFFER
*/
if (a_numargs > 0)
   {
   while(argcntr++ < a_numargs)
      {
      argtype = (MUR_ARG_TYPE)va_arg(ap, MUR_ARG_TYPE);
      bcopy((char *)&argtype, l_bufptr, sizeof(MUR_ARG_TYPE));
      l_bufptr += sizeof(MUR_ARG_TYPE);

      switch (argtype)
         {
         case MUR_LONG:
            msgargcount++;		      
            arglong = (long)va_arg(ap, long);
            bcopy((char *)&arglong, l_bufptr, sizeof(long));
            l_bufptr += sizeof(long);
            break;

         case MUR_U_LONG:
            msgargcount++;		      
            argulong = (u_long)va_arg(ap, u_long);
            bcopy((char *)&argulong, l_bufptr, sizeof(u_long));
            l_bufptr += sizeof(u_long);
            break;

         case MUR_FLOAT:
            msgargcount++;                   
            argfloat = (float)va_arg(ap, float);
            bcopy((char *)&argfloat, l_bufptr, sizeof(float));
            l_bufptr += sizeof(float);
            break;

         case MUR_STRING:
            msgargcount++;
            /* Make sure too many strings were not passed */
            if (++stringcount > MUR_NUM_STR_ARGS)
               {get_out = TRUE; break;}
            argstring = (MUR_ARGSTRING_PTR)va_arg(ap, MUR_ARGSTRING_PTR);
            bcopy((char *)argstring, l_bufptr, MUR_ARG_STRING_LEN);
            l_bufptr += MUR_ARG_STRING_LEN;
            break;

         case MUR_APPEND:
            /* Make sure only one appended text string was passed */
            if (++appendcount > MUR_NUM_APPEND)
               {get_out = TRUE; break;}
            /* Get appended text argument, but DON'T pack it yet */
            appstring = (MUR_APPTEXT_PTR)va_arg(ap, MUR_APPTEXT_PTR);  
            bcopy((char *)appstring, l_bufptr, MUR_APP_TEXT_LEN);
            l_bufptr += MUR_APP_TEXT_LEN;
            break;

         case MUR_TEXT_ONLY_MSG:
            /* Make sure this is the only argument passed */
            if (a_numargs > 1)
               {get_out = TRUE; break;}
            msgargcount++;
            textstring = (MUR_TEXTONLY_PTR)va_arg(ap, MUR_TEXTONLY_PTR);  
            bcopy((char *)textstring, l_bufptr, MUR_TEXT_ONLY_LEN);
            l_bufptr += MUR_TEXT_ONLY_LEN;
            break;

         default:
            get_out = TRUE;
            break;
         } /* switch */

      /*
      ** SEND ONLY MESSAGE CODE IF IMPROPER OR TOO MANY ARGUMENTS WERE PASSED
      */
      if ((msgargcount > MUR_NUM_ARGS) || (get_out))
         {
         l_bufptr = l_numargptr;   /* Retrieve saved position for # args */
         a_numargs = 0;            /* Don't indicate any arguments */
         bcopy((char *)&a_numargs, l_bufptr, sizeof(int));
         l_bufptr += sizeof(int);
         }

      } /* while */
   } /* if (a_numargs > 0)... */

/*
** SEND THE BUFFER DOWN THE MESSAGE QUEUE TO THE ROUTING PROCESS
*/
l_bytes = (u_long)l_bufptr - (u_long)l_buf;  /* Calculate # bytes to send */

if (global_msgq_id)
   {
   msgQSend(global_msgq_id, l_buf, l_bytes, NO_WAIT, MSG_PRI_NORMAL);
   }

} /* end: mur_route_send */


/* ==========================================================================
**
** ROUTINE: mur_route_start
*/
void mur_route_start()

/*
** DESCRIPTION
**	MUR_ROUTE_START is a VxWorks task which simply forwards messages from
**	ISP routines (sent via MUR_ROUTE_SEND calls) to the MURMUR server.
**	This task must be spawned before any user ISP routines call 
**	MUR_ROUTE_SEND, or messages will be lost.
**
**	When MUR_ROUTE_START is invoked by the user, it will first create a
**	message queue, then loop forever, listening for messages from ISP's.
**	When a message is received, MUR_ROUTE_START acts a client and makes a 
**	call to MUR_SEND, passing the message on to the MURMUR server through
**	normal means (i.e., socket I/O).
**
**	This indirect method of sending a message to the MURMUR server must
**	be used, since ISP's cannot call functions (i.e., MUR_SEND) which block
**	directly.
**
** ARGUMENTS
**	N/A
**
** VALUES RETURNED
**	N/A
**
** CALLED BY
**      User (via VxWorks shell)
**
** CALLS TO
**	bcopy
**	msgQCreate
**	msgQReceive
**	mur_internal_err
**	mur_send
**
** GLOBALS REFERENCED
**	global_msgq_id
**
** ========================================================================== 
*/
{ /* begin: mur_route_start */

MUR_ARG_STRUCT	arg[MUR_NUM_ARGS+MUR_NUM_APPEND];
unsigned long   argtype[MUR_NUM_ARGS+MUR_NUM_APPEND];
MUR_APP_TEXT	appstring;
MUR_TEXT_ONLY	textstring;
unsigned long	a_msgcode;
int		a_numargs;
MUR_ARG_TYPE	argmnttype;
int		argcntr;
char		l_buf[MUR_USER_REC_LEN];
char		*l_bufptr;
char		*l_numargptr;
long int	status;

/*
** CREATE THE MESSAGE QUEUE
*/
/* Note that only MUR_ISP_MAXMSGS can be queued up on the message queue 
   before message start getting lost */
global_msgq_id = msgQCreate(MUR_ISP_MAXMSGS, MUR_USER_REC_LEN, MSG_Q_FIFO);
if (!(global_msgq_id))
   {
   mur_internal_err(MUR_CERR_NO_MSGQ, 0, TRUE);
   return;
   }

/*
** SET THE PROCESS NAME SO EACH MESSAGE IS PREFIXED WITH "*** ROUTER ***"
*/
mur_set_proc_name("*** ROUTER ***");

/*
** LOOP FOREVER, LISTENING FOR MESSAGES FROM ISP ROUTINES 
*/
while(1)
   {
   /*
   ** WAIT FOR AN INCOMING MESSAGE ON THE MESSAGE QUEUE
   */
   status = (long int)msgQReceive(global_msgq_id, l_buf, MUR_USER_REC_LEN,
                                  WAIT_FOREVER);
   if (status == ERROR)
      {
      mur_internal_err(MUR_CERR_MSGQ_IO, 0, TRUE);
      }
   else
      {
      l_bufptr = l_buf;        /* Set buffer ptr to start of buffer */

      /*
      ** GET THE MESSAGE CODE AND NUMBER OF ARGUMENTS FROM THE BUFFER
      */
      bcopy(l_bufptr, (char *)&a_msgcode, sizeof(unsigned long));
      l_bufptr += sizeof(unsigned long);
      bcopy(l_bufptr, (char *)&a_numargs, sizeof(int));
      l_bufptr += sizeof(int);

      /*
      ** RETRIEVE ANY MESSAGE ARGUMENTS FROM THE BUFFER
      */
      if (a_numargs > 0)
         {
         argcntr = 0;  /* Reset argument counter */
         do
            {
            bcopy(l_bufptr, (char *)&argtype[argcntr], sizeof(MUR_ARG_TYPE));
            l_bufptr += sizeof(MUR_ARG_TYPE);
            argmnttype = (MUR_ARG_TYPE)argtype[argcntr];

            switch (argmnttype)
               {
               case MUR_LONG:
                  bcopy(l_bufptr, (char *)&arg[argcntr].un.longy, sizeof(long));
                  l_bufptr += sizeof(long);
                  break;

               case MUR_U_LONG:
                  bcopy(l_bufptr, (char *)&arg[argcntr].un.ulongy, sizeof(u_long));
                  l_bufptr += sizeof(u_long);
                  break;

               case MUR_FLOAT:
                  bcopy(l_bufptr, (char *)&arg[argcntr].un.floater, sizeof(float));
                  l_bufptr += sizeof(float);
                  break;

               case MUR_STRING:
                  arg[argcntr].un.stringyptr = (char *)arg[argcntr].stringy;
                  bcopy(l_bufptr, (char *)arg[argcntr].stringy, MUR_ARG_STRING_LEN);
                  arg[argcntr].stringy[MUR_ARG_STRING_LEN] = (char)0; /* Null terminate */
                  l_bufptr += MUR_ARG_STRING_LEN;
                  break;

               case MUR_APPEND:
                  arg[argcntr].un.stringyptr = (char *)appstring;
                  bcopy(l_bufptr, (char *)appstring, MUR_APP_TEXT_LEN);
                  appstring[MUR_APP_TEXT_LEN] = (char)0; /* Null terminate */
                  l_bufptr += MUR_APP_TEXT_LEN;
                  break;

               case MUR_TEXT_ONLY_MSG:
                  arg[argcntr].un.stringyptr = (char *)textstring;
                  bcopy(l_bufptr, (char *)textstring, MUR_TEXT_ONLY_LEN);
                  textstring[MUR_TEXT_ONLY_LEN] = (char)0;
                  l_bufptr += MUR_TEXT_ONLY_LEN;
                  break;

               default:
                  break;

               } /* switch */
            } while(++argcntr < a_numargs);
         }  /* if (a_numargs > 0)... */

      status = (long int)mur_send((unsigned long)a_msgcode, (int)a_numargs,
                   (MUR_ARG_TYPE)argtype[0], (unsigned long)arg[0].un.generic,
                   (MUR_ARG_TYPE)argtype[1], (unsigned long)arg[1].un.generic,
                   (MUR_ARG_TYPE)argtype[2], (unsigned long)arg[2].un.generic,
                   (MUR_ARG_TYPE)argtype[3], (unsigned long)arg[3].un.generic,
                   (MUR_ARG_TYPE)argtype[4], (unsigned long)arg[4].un.generic,
                   (MUR_ARG_TYPE)argtype[5], (unsigned long)arg[5].un.generic,
                   (MUR_ARG_TYPE)argtype[6], (unsigned long)arg[6].un.generic,
                   (MUR_ARG_TYPE)argtype[7], (unsigned long)arg[7].un.generic,
                   (MUR_ARG_TYPE)argtype[8], (unsigned long)arg[8].un.generic);

      } /* else (from: if (status == ERROR)...) */
   } /* while(1)... */
} /* end: mur_route_start */


/* ==========================================================================
**
** MACRO: mur_route_send_text
**
** DESCRIPTION
** 	MUR_ROUTE_SEND_TEXT is a macro which invokes MUR_ROUTE_SEND to send a
**      message containing text only to the MURMUR server.
** 
** ARGUMENTS
**      a_text
**	    Type:        char[132]
**	    Access:      read only
**	    Mechanism:   by reference
**	    Description: The text string message to be reported.
**
** VALUES RETURNED
**	MUR_SUCCESS      - Message sent to MURMUR server successfully.
**	MUR_SEV_ERR      - Unable to send message to MURMUR server.
**
** CALLED BY
**      user's application
**
** ========================================================================== 
*/
#endif  /* MUR_VXWORKS */
