/*************************************************
 * CVS Log
 * mur_f_user.c,v
 * Revision 1.1.1.1  1993/07/14  19:43:23  dbrenner
 * UNIDAQ 2.1 distribution - July 14,1993
 *
*************************************************/
/* ==========================================================================
**
** MUR_C_USER.C
**
** DESCRIPTION
**	This file contains all the Fortran interface routines to the 
**	"C" routines which are available to the client library.
**
** DEVELOPERS
**	Lourdu R. Udumula
**
**	Fermilab Data Acquisition Group
**	Batavia, Il 60510, U.S.A.
**
** 
** ENTRY POINTS			 SCOPE
**	mur_send_f		external AIX VMS only
**	mur_set_proc_name_f	external AIX VMS only
**	mur_send_text_f		external AIX VMS only
**	mur_open_f		external AIX VMS only
**	mur_send_text_f		external AIX VMS only
**	mur_msg_facility_f	external AIX VMS only
**	mur_msg_severity_f	external AIX VMS only
**	mur_msg_number_f	external AIX VMS only
**	mur_msg_success_f	external AIX VMS only
**	mur_msg_strip_sev_f	external AIX VMS only
**
**	mur_send_f_		external not for AIX and VMS
**	mur_set_proc_name_f_	external
**	mur_send_text_f_	external
**	mur_open_f_		external
**	mur_send_text_f_	external
**	mur_msg_facility_f_	external 
**	mur_msg_severity_f_	external
**	mur_msg_number_f_	external
**	mur_msg_success_f_	external
**	mur_msg_strip_sev_f_	external
**
** NOTE: varargs.h is used instead of stdarg.h on all the flavors.
** 	 va_arg macro in the stdarg.h does not work on all the flavors and
** 	 this is being avoided. The va_arg in the varargs.h works for all
**	 the flavors and hence is being used. If varargs is not supported on
**	 a particular flavor then it should be conditionalized for that flavor
**	 and stdarg should be used. This affects only the mur_send_f and the
**	 mur_send_f_ routines in this file. 
**	----------------------------------------------------------------------
**
** 	These routines give entry points to fortran programs. Since the 
**	fortran compilers behave differently on different flavors, There are
**	two entry points for each routine. Either entry point can be used
**	by the fortran program. The fortran compilers on the AIX and VMS 
**	generate entry points as the same name as the calling routine. The
** 	fortran compiler on SunOS,IRIX,ULTRIX generate entry points for the
**	calling routines with an underscore suffix. 
**	These wrapper routines are required since the fortan functions pass
**	arguments by reference and hece  have to converted to a form the 
**	c routines for murmur_client understand.
*/

#include "mur_cond.h"

#ifdef MUR_UNIX
#include <varargs.h>	/* Should use stdarg.h in future. Right now stdarg.h*/
			/* does not work on all flavors */
#include <stdio.h>	/* Need for SGI (__file_s declaration) */
#include <rpc/types.h>
#include <rpc/xdr.h>
#include "murmur_msg_c.h"
#endif /* MUR_UNIX */

#ifdef MUR_VMS
#include <varargs.h>	/* Should use stdarg.h in future. Right now stdarg.h*/
			/* does not work on all flavors */
#include "MULTINET_COMMON_ROOT:[multinet.include.rpc]rpc.h"
#include "MULTINET_COMMON_ROOT:[multinet.include.rpc]xdr.h"
#include <descrip.h>
#include <iodef.h>
#endif

#include "mur_def.h"


/* -----------------------------------------------------------------*/
/* Entry point for mur_send_f_ on all flavors only 		    */
/* ON AIX if the -qexternal switch is used on the fortran compiler  */
/* -----------------------------------------------------------------*/

unsigned long mur_send_f_(va_alist)
va_dcl
{ 
va_list			ap;
unsigned long		*a_msgcode;
int			*a_numargs;


MUR_ARG_STRUCT  	arg[MUR_NUM_ARGS+MUR_NUM_APPEND];
unsigned long   	argtype[MUR_NUM_ARGS+MUR_NUM_APPEND];
int			argcntr	    = 0;
int	        	msgargcount = 0;
int			appendcount = 0;
int	        	stringcount = 0;
MUR_BOOL		get_out = FALSE;
unsigned long		*argmnttype;
long			*arglong;
unsigned long		*argulong;
double			*argfloat;
MUR_ARGSTRING_PTR 	argstring;
MUR_APPTEXT_PTR	  	appstring;
MUR_TEXTONLY_PTR  	textstring;
unsigned long 		status;
int			numargs;

#ifdef MUR_VMS
struct dsc$descriptor_s *str;	/* ON vms strings are passed as this struct */
unsigned short	len;
#endif


va_start(ap);
a_msgcode = va_arg(ap, unsigned long *);
a_numargs = va_arg(ap, int *);

/*
   SAVE ANY MESSAGE ARGUMENTS IN THE BUFFER
   ---------------------------------------- */
if (*a_numargs > 0)
   {
   while(argcntr < *a_numargs)
      {
      argmnttype = (unsigned long *) va_arg(ap, unsigned long *);
      bcopy(argmnttype,(char *)&argtype[argcntr], sizeof(MUR_ARG_TYPE));

      switch (*argmnttype)
         {
         case MUR_LONG:
            msgargcount++;		      
            arglong = (long *)va_arg(ap, long *);
	    bcopy(arglong, (char *)&arg[argcntr].un.longy, sizeof(long));
            break;

         case MUR_U_LONG:
            msgargcount++;		      
            argulong = (u_long *)va_arg(ap, u_long *);
	    bcopy(argulong, (char *)&arg[argcntr].un.ulongy, sizeof(u_long));
            break;

         case MUR_FLOAT:
            msgargcount++;                   
            argfloat = (double *)va_arg(ap, double *);
	    bcopy(argfloat, (char *)&arg[argcntr].un.floater, sizeof(double));
            break;

         case MUR_STRING:
	    arg[argcntr].un.stringyptr = (char *)arg[argcntr].stringy;
            msgargcount++;
            /* Make sure too many strings were not passed */
            if (++stringcount > MUR_NUM_STR_ARGS)
               {get_out = TRUE; break;}
#ifdef MUR_VMS
	    str=(struct dsc$descriptor_s *)va_arg(ap,struct dsc$descriptor_s *);
	    memcpy(arg[argcntr].stringy,str->dsc$a_pointer,MUR_ARG_STRING_LEN);
#else
            argstring = (MUR_ARGSTRING_PTR)va_arg(ap, MUR_ARGSTRING_PTR);
	    bcopy(argstring, (char *)arg[argcntr].stringy, MUR_ARG_STRING_LEN);
#endif
	    arg[argcntr].stringy[MUR_ARG_STRING_LEN] = (char )0;
            break;

         case MUR_APPEND:
            /* Make sure only one appended text string was passed */
            if (++appendcount > MUR_NUM_APPEND)
               {get_out = TRUE; break;}
#ifdef MUR_VMS
	    str = (struct dsc$descriptor_s *) va_arg(ap,struct dsc$descriptor_s *);
	    appstring = str->dsc$a_pointer;
#else
            /* Get appended text argument, but DON'T pack it yet */
            appstring = (MUR_APPTEXT_PTR)va_arg(ap, MUR_APPTEXT_PTR);  
#endif
	    arg[argcntr].un.stringyptr = (char *)appstring;
            break;

         case MUR_TEXT_ONLY_MSG:
            /* Make sure this is the only argument passed */
            if (*a_numargs > 1)
               {get_out = TRUE; break;}
            msgargcount++;
#ifdef MUR_VMS
	    str = (struct dsc$descriptor_s *) va_arg(ap,struct dsc$descriptor_s *);
	    textstring = (char *) str->dsc$a_pointer;
#else
            textstring = (MUR_TEXTONLY_PTR)va_arg(ap, MUR_TEXTONLY_PTR);  
#endif
	    arg[argcntr].un.stringyptr = (char *)textstring;
            break;

         default:
            get_out = TRUE;
            break;
         } /* switch */

      /*
      ** SEND ONLY MESSAGE CODE IF IMPROPER OR TOO MANY ARGUMENTS WERE PASSED
      */
      if ((msgargcount > MUR_NUM_ARGS) || (get_out))
         {
         *a_numargs = 0;            /* Don't indicate any arguments */
         }
	argcntr++;

      } /* while */
   } /* if (a_numargs > 0)... */

	status = mur_send((unsigned long)*a_msgcode, (int)*a_numargs,
                   (MUR_ARG_TYPE)argtype[0], (unsigned long)arg[0].un.generic,
                   (MUR_ARG_TYPE)argtype[1], (unsigned long)arg[1].un.generic,
                   (MUR_ARG_TYPE)argtype[2], (unsigned long)arg[2].un.generic,
                   (MUR_ARG_TYPE)argtype[3], (unsigned long)arg[3].un.generic,
                   (MUR_ARG_TYPE)argtype[4], (unsigned long)arg[4].un.generic,
                   (MUR_ARG_TYPE)argtype[5], (unsigned long)arg[5].un.generic,
                   (MUR_ARG_TYPE)argtype[6], (unsigned long)arg[6].un.generic,
                   (MUR_ARG_TYPE)argtype[7], (unsigned long)arg[7].un.generic,
                   (MUR_ARG_TYPE)argtype[8], (unsigned long)arg[8].un.generic);

	return (status);
}



#if defined(_AIX) || defined (MUR_VMS)

unsigned long mur_send_f(va_alist)
va_dcl
{ 
va_list			ap;
unsigned long		*a_msgcode;
int			*a_numargs;


MUR_ARG_STRUCT  	arg[MUR_NUM_ARGS+MUR_NUM_APPEND];
unsigned long   	argtype[MUR_NUM_ARGS+MUR_NUM_APPEND];
int			argcntr	    = 0;
int	        	msgargcount = 0;
int			appendcount = 0;
int	        	stringcount = 0;
MUR_BOOL		get_out = FALSE;
unsigned long		*argmnttype;
long			*arglong;
unsigned long		*argulong;
double			*argfloat;
MUR_ARGSTRING_PTR 	argstring;
MUR_APPTEXT_PTR	  	appstring;
MUR_TEXTONLY_PTR  	textstring;
unsigned long 		status;
int			numargs;

#ifdef MUR_VMS
struct dsc$descriptor_s *str;	/* ON vms strings are passed as this struct */
unsigned short	len;
#endif


va_start(ap);
a_msgcode = va_arg(ap, unsigned long *);
a_numargs = va_arg(ap, int *);

/*
   SAVE ANY MESSAGE ARGUMENTS IN THE BUFFER
   ---------------------------------------- */
if (*a_numargs > 0)
   {
   while(argcntr < *a_numargs)
      {
      argmnttype = (unsigned long *) va_arg(ap, unsigned long *);
      bcopy(argmnttype,(char *)&argtype[argcntr], sizeof(MUR_ARG_TYPE));

      switch (*argmnttype)
         {
         case MUR_LONG:
            msgargcount++;		      
            arglong = (long *)va_arg(ap, long *);
	    bcopy(arglong, (char *)&arg[argcntr].un.longy, sizeof(long));
            break;

         case MUR_U_LONG:
            msgargcount++;		      
            argulong = (u_long *)va_arg(ap, u_long *);
	    bcopy(argulong, (char *)&arg[argcntr].un.ulongy, sizeof(u_long));
            break;

         case MUR_FLOAT:
            msgargcount++;                   
            argfloat = (double *)va_arg(ap, double *);
	    bcopy(argfloat, (char *)&arg[argcntr].un.floater, sizeof(double));
            break;

         case MUR_STRING:
	    arg[argcntr].un.stringyptr = (char *)arg[argcntr].stringy;
            msgargcount++;
            /* Make sure too many strings were not passed */
            if (++stringcount > MUR_NUM_STR_ARGS)
               {get_out = TRUE; break;}
#ifdef MUR_VMS
	    str=(struct dsc$descriptor_s *)va_arg(ap,struct dsc$descriptor_s *);
	    memcpy(arg[argcntr].stringy,str->dsc$a_pointer,MUR_ARG_STRING_LEN);
#else
            argstring = (MUR_ARGSTRING_PTR)va_arg(ap, MUR_ARGSTRING_PTR);
	    bcopy(argstring, (char *)arg[argcntr].stringy, MUR_ARG_STRING_LEN);
#endif
	    arg[argcntr].stringy[MUR_ARG_STRING_LEN] = (char )0;
            break;

         case MUR_APPEND:
            /* Make sure only one appended text string was passed */
            if (++appendcount > MUR_NUM_APPEND)
               {get_out = TRUE; break;}
#ifdef MUR_VMS
	    str = (struct dsc$descriptor_s *) va_arg(ap,struct dsc$descriptor_s *);
	    appstring = str->dsc$a_pointer;
#else
            /* Get appended text argument, but DON'T pack it yet */
            appstring = (MUR_APPTEXT_PTR)va_arg(ap, MUR_APPTEXT_PTR);  
#endif
	    arg[argcntr].un.stringyptr = (char *)appstring;
            break;

         case MUR_TEXT_ONLY_MSG:
            /* Make sure this is the only argument passed */
            if (*a_numargs > 1)
               {get_out = TRUE; break;}
            msgargcount++;
#ifdef MUR_VMS
	    str = (struct dsc$descriptor_s *) va_arg(ap,struct dsc$descriptor_s *);
	    textstring = (char *) str->dsc$a_pointer;
#else
            textstring = (MUR_TEXTONLY_PTR)va_arg(ap, MUR_TEXTONLY_PTR);  
#endif
	    arg[argcntr].un.stringyptr = (char *)textstring;
            break;

         default:
            get_out = TRUE;
            break;
         } /* switch */

      /*
      ** SEND ONLY MESSAGE CODE IF IMPROPER OR TOO MANY ARGUMENTS WERE PASSED
      */
      if ((msgargcount > MUR_NUM_ARGS) || (get_out))
         {
         *a_numargs = 0;            /* Don't indicate any arguments */
         }
	argcntr++;

      } /* while */
   } /* if (a_numargs > 0)... */

	status = mur_send((unsigned long)*a_msgcode, (int)*a_numargs,
                   (MUR_ARG_TYPE)argtype[0], (unsigned long)arg[0].un.generic,
                   (MUR_ARG_TYPE)argtype[1], (unsigned long)arg[1].un.generic,
                   (MUR_ARG_TYPE)argtype[2], (unsigned long)arg[2].un.generic,
                   (MUR_ARG_TYPE)argtype[3], (unsigned long)arg[3].un.generic,
                   (MUR_ARG_TYPE)argtype[4], (unsigned long)arg[4].un.generic,
                   (MUR_ARG_TYPE)argtype[5], (unsigned long)arg[5].un.generic,
                   (MUR_ARG_TYPE)argtype[6], (unsigned long)arg[6].un.generic,
                   (MUR_ARG_TYPE)argtype[7], (unsigned long)arg[7].un.generic,
                   (MUR_ARG_TYPE)argtype[8], (unsigned long)arg[8].un.generic);

	return (status);
}
#endif /* AIX or VMS */

/* ---------------------------------------------------------------- */
/* ENTRY POINT  mur_set_proc_name_f 				    */
/* Conditionalized for VMS and AIX flavors			    */
/* ---------------------------------------------------------------- */
#if defined(_AIX) || defined (MUR_VMS)

#ifdef MUR_STDC

#if defined(MUR_VMS)
unsigned long mur_set_proc_name_f(struct dsc$descriptor_s *str)
#else
unsigned long mur_set_proc_name_f(char *name)
#endif

#else

#if defined(MUR_VMS)
unsigned long mur_set_proc_name_f(str)
struct dsc$descriptor_s *str;
#else
unsigned long mur_set_proc_name_f(name)
char *name;
#endif

#endif	/* MUR_STDC */

	{

#if defined(MUR_VMS)
	char	name[MUR_PREFIX_LEN];
	memcpy(&name[0],str->dsc$a_pointer, MUR_PREFIX_LEN);
	name[MUR_PREFIX_LEN] = (char ) 0;
#endif

	return (mur_set_proc_name(name));
	}

#endif /* _AIX or VMS */


/* ---------------------------------------------------------------- */
/* ENTRY POINT mur_set_proc_name_f_ 				    */
/* For all flavors.						    */
/* ---------------------------------------------------------------- */
#ifdef MUR_STDC

#if defined(MUR_VMS)
unsigned long mur_set_proc_name_f_(struct dsc$descriptor_s *str)
#else
unsigned long mur_set_proc_name_f_(char *name)
#endif

#else

#if defined(MUR_VMS)
unsigned long mur_set_proc_name_f_(str)
struct dsc$descriptor_s *str;
#else
unsigned long mur_set_proc_name_f_(name)
char *name;
#endif

#endif	/* MUR_STDC */

	{
#if defined(MUR_VMS)
	char	name[MUR_PREFIX_LEN];
	memcpy(&name[0],str->dsc$a_pointer, MUR_PREFIX_LEN);
	name[MUR_PREFIX_LEN] = (char ) 0;
#endif

	return (mur_set_proc_name(name));
	}


/* ------------------------------------------------------- */
/* ENTRY POINT mur_open_f				   */
/* Conditionalized for AIX and VMS flavors only 	   */
/* ------------------------------------------------------- */
#if defined(_AIX) || defined (MUR_VMS)
#ifdef MUR_STDC
unsigned long mur_open_f(void)
#else
unsigned long mur_open_f()
#endif
	{
	return (mur_open());
	}
#endif

/* --------------------------------------------------------------- */
/* ENTRY POINT mur_open_f_					   */
/* For all flavors						   */
/* --------------------------------------------------------------- */
#ifdef MUR_STDC
unsigned long mur_open_f_(void)
#else
unsigned long mur_open_f_()
#endif
	{
	return (mur_open());	
	}

/* --------------------------------------------------------------- */
/* The following are macros for c interface 			   */
/* --------------------------------------------------------------- */

#if defined(_AIX) || defined (MUR_VMS)
#ifdef MUR_STDC
unsigned long mur_msg_facility_f(unsigned long *msgcode)
#else
unsigned long mur_msg_facility_f(msgcode)
unsigned long *msgcode;
#endif
	{
	return ( (*msgcode & MUR_M_FAC) >> MUR_LSB_FAC);
	}
#endif /* AIX or MUR_VMS */

#ifdef MUR_STDC
unsigned long mur_msg_facility_f_(unsigned long *msgcode)
#else
unsigned long mur_msg_facility_f_(msgcode)
unsigned long *msgcode;
#endif
	{
	return ( (*msgcode & MUR_M_FAC) >> MUR_LSB_FAC);
	}

#if defined(_AIX) || defined (MUR_VMS)
#ifdef MUR_STDC
unsigned long mur_msg_number_f(unsigned long *msgcode)
#else
unsigned long mur_msg_number_f(msgcode)
unsigned long *msgcode;
#endif
	{
	return ( (*msgcode & MUR_M_MSG) >> MUR_LSB_MSG);
	}
#endif /* AIX or MUR_VMS */

#ifdef MUR_STDC
unsigned long mur_msg_number_f_(unsigned long *msgcode)
#else
unsigned long mur_msg_number_f_(msgcode)
unsigned long *msgcode;
#endif
	{
	return ( (*msgcode & MUR_M_MSG) >> MUR_LSB_MSG);
	}

#if defined(_AIX) || defined (MUR_VMS)
#ifdef MUR_STDC
unsigned long mur_msg_severity_f(unsigned long *msgcode)
#else
unsigned long mur_msg_severity_f(msgcode)
unsigned long *msgcode;
#endif
	{
	return (*msgcode & MUR_M_SEV); 
	}
#endif

#ifdef MUR_STDC
unsigned long mur_msg_severity_f_(unsigned long *msgcode)
#else
unsigned long mur_msg_severity_f_(msgcode)
unsigned long *msgcode;
#endif
	{
	return (*msgcode & MUR_M_SEV); 
	}

#if defined(_AIX) || defined (MUR_VMS)
#ifdef MUR_STDC
unsigned long mur_msg_success_f(unsigned long *msgcode)
#else
unsigned long mur_msg_success_f(msgcode)
unsigned long *msgcode;
#endif
	{
	return (*msgcode & MUR_M_SUC); 
	}
#endif

#ifdef MUR_STDC
unsigned long mur_msg_success_f_(unsigned long *msgcode)
#else
unsigned long mur_msg_success_f_(msgcode)
unsigned long *msgcode;
#endif
	{
	return (*msgcode & MUR_M_SUC); 
	}

/* ---------------------------------------------------------------- */
/* ENTRY POINT  mur_msg_fail_f					    */
/* Conditionalized for VMS and AIX flavors			    */
/* ---------------------------------------------------------------- */
#if defined(_AIX) || defined (MUR_VMS)
#ifdef MUR_STDC
unsigned long mur_msg_fail_f(unsigned long *msgcode)
#else
unsigned long mur_msg_fail_f(msgcode)
unsigned long *msgcode;
#endif
	{
	return (!(*msgcode & MUR_M_SUC));
	}
#endif /* AIX or VMS */

/* ---------------------------------------------------------------- */
/* ENTRY POINT  mur_msg_fail_f_					    */
/* For all flavors			   			    */
/* ---------------------------------------------------------------- */
#ifdef MUR_STDC
unsigned long mur_msg_fail_f_(unsigned long *msgcode)
#else
unsigned long mur_msg_fail_f_(msgcode)
unsigned long *msgcode;
#endif
	{
	return (!(*msgcode & MUR_M_SUC));
	}

/* ---------------------------------------------------------------- */
/* ENTRY POINT  mur_strip_sev_f				    */
/* Conditionalized for VMS and AIX flavors			    */
/* ---------------------------------------------------------------- */
#if defined(_AIX) || defined (MUR_VMS)
#ifdef MUR_STDC
unsigned long mur_msg_strip_sev_f(unsigned long *msgcode)
#else
unsigned long mur_msg_strip_sev_f(msgcode)
unsigned long *msgcode;
#endif
	{
	return (*msgcode & MUR_M_NOSEV); 
	}
#endif

/* ---------------------------------------------------------------- */
/* ENTRY POINT  mur_msg_strip_f_				    */
/* Conditionalized for all flavors			    	    */
/* ---------------------------------------------------------------- */
#ifdef MUR_STDC
unsigned long mur_msg_strip_sev_f_(unsigned long *msgcode)
#else
unsigned long mur_msg_strip_sev_f_(msgcode)
unsigned long *msgcode;
#endif
	{
	return (*msgcode & MUR_M_NOSEV); 
	}

/* ---------------------------------------------------------------- */
/* ENTRY POINT  mur_send_text_f 				    */
/* Conditionalized for VMS and AIX flavors			    */
/* ---------------------------------------------------------------- */
#if defined(_AIX) || defined (MUR_VMS)

#ifdef MUR_STDC

#if defined(MUR_VMS)
unsigned long mur_send_text_f(struct dsc$descriptor_s *str)
#else
unsigned long mur_send_text_f(char *name)
#endif

#else

#if defined(MUR_VMS)
unsigned long mur_send_text_f(str)
struct dsc$descriptor_s *str;
#else
unsigned long mur_send_text_f(name)
char *name;
#endif

#endif	/* MUR_STDC */

	{

#if defined(MUR_VMS)
	char	name[MUR_TEXT_ONLY_LEN];
	memcpy(&name[0],str->dsc$a_pointer, MUR_TEXT_ONLY_LEN);
	name[MUR_TEXT_ONLY_LEN] = (char ) 0;
#endif

	return (mur_send_text(name));
	}

#endif /* _AIX or VMS */



/* ---------------------------------------------------------------- */
/* ENTRY POINT  mur_send_text_f_ 				    */
/* Conditionalized for VMS and AIX flavors			    */
/* ---------------------------------------------------------------- */
#ifdef MUR_STDC

#if defined(MUR_VMS)
unsigned long mur_send_text_f_(struct dsc$descriptor_s *str)
#else
unsigned long mur_send_text_f_(char *name)
#endif

#else

#if defined(MUR_VMS)
unsigned long mur_send_text_f_(str)
struct dsc$descriptor_s *str;
#else
unsigned long mur_send_text_f_(name)
char *name;
#endif

#endif	/* MUR_STDC */

	{

#if defined(MUR_VMS)
	char	name[MUR_TEXT_ONLY_LEN];
	memcpy(&name[0],str->dsc$a_pointer, MUR_TEXT_ONLY_LEN);
	name[MUR_TEXT_ONLY_LEN] = (char ) 0;
#endif

	return (mur_send_text(name));
	}

/* ---------------------------------------------------------------- */
/* ENTRY POINT  mur_translate_vms_f				    */
/* Conditionalized for VMS and AIX flavors			    */
/* ---------------------------------------------------------------- */
#if defined(_AIX) || defined (MUR_VMS)
#ifdef MUR_STDC
unsigned long mur_translate_vms_f(unsigned long *vms_code, unsigned long *mur_code)
#else
unsigned long mur_translate_vms_f(vms_code, mur_code)
unsigned long *vms_code;
unsigned long *mur_code;
#endif
	{
	return (mur_translate_vms(*vms_code,mur_code));
	}
#endif /* AIX or VMS */


/* ---------------------------------------------------------------- */
/* ENTRY POINT  mur_translate_vms_f_				    */
/* For all flavors			    			    */
/* ---------------------------------------------------------------- */

#ifdef MUR_STDC
unsigned long mur_translate_vms_f_(unsigned long *vms_code, unsigned long *mur_code)
#else
unsigned long mur_translate_vms_f_(vms_code, mur_code)
unsigned long *vms_code;
unsigned long *mur_code;
#endif
	{
	return (mur_translate_vms(*vms_code,mur_code));
	}


/* ---------------------------------------------------------------- */
/* ENTRY POINT  mur_translate_unix_f				    */
/* Conditionalized for VMS and AIX flavors			    */
/* ---------------------------------------------------------------- */
#if defined(_AIX) || defined (MUR_VMS)
#ifdef MUR_STDC
unsigned long mur_translate_unix_f(int *errno_code, unsigned long *mur_code)
#else
unsigned long mur_translate_unix_f(errno_code, mur_code)
int  *errno_code;
unsigned long *mur_code;
#endif
	{
	return (mur_translate_unix(*errno_code, mur_code));
	}
#endif /* AIX or VMS */

/* ---------------------------------------------------------------- */
/* ENTRY POINT  mur_translate_unix_f_				    */
/* For all flavors			    			    */
/* ---------------------------------------------------------------- */
#ifdef MUR_STDC
unsigned long mur_translate_unix_f_(int *errno_code, unsigned long *mur_code)
#else
unsigned long mur_translate_unix_f_(errno_code, mur_code)
int  *errno_code;
unsigned long *mur_code;
#endif
	{
	return (mur_translate_unix(*errno_code, mur_code));
	}

