/*************************************************
 * CVS Log
 * mur_send_message.c,v
 * Revision 1.1.1.1  1993/07/14  19:43:23  dbrenner
 * UNIDAQ 2.1 distribution - July 14,1993
 *
*************************************************/
/*
================================================================================

  mur_send_message.c - A command line utility to send messages to a murmur
  server (specified by the environment variable MURMUR_SERVER_ADD). You can 
  send a message code (specified in hex), or a pure text message with an 
  optional repeat count. Usage:

  mur_send_message [-n repeat-count] [-c code] [-t text] [-u errno] [-v vmscode]

  you must specify either -t, -c, -v, or -u , -n is optional and defaults to 1. 
  You can specify -t with -u, -v or -c together, but cannot specify any 
  combination of -c, -u, and -v together.

  Gene Oleynik
  OLS
  12-Oct-1992

  Modifications:
  13-Oct-1992	CLM	Added the -v and -u options for specifying
			vms and unix system messages.

================================================================================
*/


#include "mur_macros.h"
#include "mur_user.h"
#include "murmur_msg_c.h"

void	usage (s) 
	char *s;
	{
  	   printf("Usage: %s [-n count] [-c message-code] [-t text]",s);
	   printf("[-u errno-code] [-v VMS-code]\n");
	   exit (0);
	}

main(argc, argv)
int argc;
char * argv[];
{
char inbuf;
int repeat = 1;
unsigned long code = 0;
char text[256+1];
unsigned long status;
extern char *optarg;
int valid;

valid = 0;

text[0]=0;


if(argc < 2)
   {
   usage(argv[0]);
   exit(0);
   }

while((inbuf = getopt(argc,argv,"n:c:t:u:v:")) != ((char) -1))
   {
      switch (inbuf)
         {
         case 'n':
             repeat = (int) strtol(optarg,0,10);
             break;
         case 'c':
	     valid =1;
            code = (int) strtol(optarg,0,16);
            break;
         case 't':
	    valid = 1;
            strncpy(text,optarg,256);
            text[256] = 0;
            break; 
	 case 'u':
	    valid = 1;
	    status = mur_translate_unix((int)strtol(optarg,0,10),&code);
	    if (status != MUR_SUCCESS) exit(0);
	    break;
	 case 'v':
	    valid = 1;
	    status = mur_translate_vms((int)strtol(optarg,0,16),&code);
	    if (status != MUR_SUCCESS) exit(0);
	    break;
	 default : 
	    usage(argv[0]);
	    exit(0);
	    break;
          }
      }
/* must have at least -t or -c */

if(!valid)
   {
   usage(argv[0]);
   exit(0);
   }


for(;repeat;repeat--)
   {
   if(code)mur_send(code,0);
   if(strlen(text))mur_send(0,1,MUR_TEXT_ONLY_MSG,&text[0]);
   }
exit(0);
}
