/*************************************************
 * ipc_msg_svc.c,v
*************************************************/
/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#include <stdio.h>
#include <rpc/rpc.h>
#include "ipc_msg.h"

static void messageprog_1();

#ifdef VXWORKS
msg_server()
#else
main()
#endif
{
	SVCXPRT *transp;

#ifdef VXWORKS
        rpcTaskInit();
#endif
	pmap_unset(MESSAGEPROG, MESSAGEVERS);

#ifdef VXWORKS
	transp = svcudp_create(RPC_ANYSOCK,200,200);
#else
	transp = svcudp_create(RPC_ANYSOCK);
#endif
	if (transp == NULL) {
		fprintf(stderr, "cannot create udp service.\n");
		exit(1);
	}
	if (!svc_register(transp, MESSAGEPROG, MESSAGEVERS, messageprog_1, IPPROTO_UDP)) {
		fprintf(stderr, "unable to register (MESSAGEPROG, MESSAGEVERS, udp).\n");
		exit(1);
	}

	transp = svctcp_create(RPC_ANYSOCK, 0, 0);
	if (transp == NULL) {
		fprintf(stderr, "cannot create tcp service.\n");
		exit(1);
	}
	if (!svc_register(transp, MESSAGEPROG, MESSAGEVERS, messageprog_1, IPPROTO_TCP)) {
		fprintf(stderr, "unable to register (MESSAGEPROG, MESSAGEVERS, tcp).\n");
		exit(1);
	}
	svc_run();
	fprintf(stderr, "svc_run returned\n");
	exit(1);
}

static void
messageprog_1(rqstp, transp)
	struct svc_req *rqstp;
	SVCXPRT *transp;
{
	union {
		char *sendmessage_1_arg;
		char *checkprocess_1_arg;
		char *startprocess_1_arg;
		struct rsp_send readsp_1_arg;
		struct rsp_send_all readspall_1_arg;
	} argument;
	char *result;
	bool_t (*xdr_argument)(), (*xdr_result)();
	char *(*local)();

	switch (rqstp->rq_proc) {
	case NULLPROC:
		svc_sendreply(transp, xdr_void, NULL);
		return;

	case SENDMESSAGE:
		xdr_argument = xdr_wrapstring;
		xdr_result = xdr_int;
		local = (char *(*)()) sendmessage_1;
		break;

	case CHECKPROCESS:
		xdr_argument = xdr_wrapstring;
		xdr_result = xdr_int;
		local = (char *(*)()) checkprocess_1;
		break;

	case STARTPROCESS:
		xdr_argument = xdr_wrapstring;
		xdr_result = xdr_int;
		local = (char *(*)()) startprocess_1;
		break;

	case READSP:
		xdr_argument = xdr_rsp_send;
		xdr_result = xdr_rsp_ret;
		local = (char *(*)()) readsp_1;
		break;

	case READSPALL:
		xdr_argument = xdr_rsp_send_all;
		xdr_result = xdr_rsp_ret_all;
		local = (char *(*)()) readspall_1;
		break;

	default:
		svcerr_noproc(transp);
		return;
	}
	memset(&argument, 0, sizeof(argument));
	if (!svc_getargs(transp, xdr_argument, &argument)) {
		svcerr_decode(transp);
		return;
	}
	result = (*local)(&argument, rqstp);
	if (result != NULL && !svc_sendreply(transp, xdr_result, result)) {
		svcerr_systemerr(transp);
	}
	if (!svc_freeargs(transp, xdr_argument, &argument)) {
		fprintf(stderr, "unable to free arguments\n");
		exit(1);
	}
}

