/**************************************************
 program VMEdo: executes single VME command
 author C. Timmermans, University of Michigan
 ***************************************************/
/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/
#include <stdlib.h>
#include <unistd.h>

#define EIGHT_BITS 1
#define SIXTEEN_BITS 2

#ifdef BIT3
#include<sys/file.h>
#include<sys/ioctl.h>
#include<sys/btio.h>
#endif
#ifdef DEC
#include "vmelib.h"
#endif

#ifdef BIT3
char devname[100];
#endif
#ifdef DEC
int devtype;
int devmodes[] = {
  VME_A16D16MODE,VME_A16D32MODE,VME_A24D16MODE,
  VME_A24D32MODE,VME_A32D16MODE,VME_A32D32MODE};
int start_addr;
int map_size;
#endif
int i_dev;
int rw_opt = -1;
char rw_str[][10] = {"read","read","write","write"};
int address,n_words,wsiz;
char *buf = NULL;
char tmp[50];

main()
{
  int i_opt;

  vme_info();
  vme_open();
  while(1){
    if((i_opt = get_option()) < 0 ) break;
    if(i_opt == 0) get_parameters();
    execute_command();
  }
  vme_close();
}

get_option()
{
  int i;
  while(1){
    printf("***** Please select one of the following: *****\n");
    printf(" Read one or more short words (8 bits) from VME ...... 1\n");
    printf(" Read one or more words (16 bits) from VME ........... 2\n");
    printf(" Write one or more short words (8 bits) to VME ....... 3\n");
    printf(" Write one or more words (16 bits) to VME ............ 4\n");
    printf(" Repeat previous command ............................. 5\n");
    printf(" Exit program ........................................ 6\n");
    printf("Selection >> ");
    scanf("%s",tmp);
    if(sscanf(tmp,"%d",&i) != 0) {
      if(i != 5) rw_opt = i;
      if(i == 5 && rw_opt >= 0) return(1); /*** Do not ask parameters ***/
      if((rw_opt > 0) && (rw_opt < 7)) break;
    }
  }
  rw_opt -=1;
  if(rw_opt == 5) return(-1); /*** Exit the program ***/
  else return(0);             /*** Ask for additional parameters ***/
}

get_parameters()
{
  short s;
  int i;

/*** First of all get the start address from where to read/write ***/
  printf(" Give starting address (hex)\n");
  i = 0;
  while(i == 0){
    printf("address >> ");
    scanf("%s",tmp);
    if(sscanf(tmp,"%x",&address) == 1) i = 1;
  }

/*** secondly get to know how much to read/write ***/
  n_words = 0;
  while ( n_words <= 0){
    if((rw_opt%2) == 1){
      wsiz = SIXTEEN_BITS;
      printf(" How many words (16 bits) do you want to %s\nwords >> ",
	     rw_str[rw_opt]);
    }
    else{
      wsiz = EIGHT_BITS;
      printf(" How many short words (8 bits) do you want to %s\nwords >> ",
	     rw_str[rw_opt]);
    }
    scanf("%s",tmp);
    sscanf(tmp,"%d",&n_words);
  }
/*** free the bufferspace  ***/
  if(buf != NULL) free(buf);
/*** Allocate memory to hold this data ***/
  if((rw_opt%2) == 1) buf = malloc(n_words*sizeof(short));
  else                buf = malloc(n_words*sizeof(char));
  if(rw_opt >=2) {
/*** get all the values to write in VME ***/
    printf(" Enter the values (hex) one by one\n");
    i = 0;
/*** Make sure to ask the right number of words ***/
    if(rw_opt == 3) n_words = n_words*2;
/*** Ask word by word ***/
    while(i < n_words){
      printf(" value >> ");
      scanf("%s",tmp);
      if(sscanf(tmp,"%hx",&s) == 1){
	if(rw_opt == 2) buf[i++] = s&0xff;
	else {
	  memcpy(&(buf[i]),&s,2);
	  i+=2;
	}
      }
      else printf(" ERROR!! repeat ");
    }
/*** Make sure to write the right number of words ***/
    if(rw_opt == 3) n_words = n_words/2;
  }
}

execute_command()
{
  int i;
  short s;

/*** perform the reading or writing ***/
  if(rw_opt >=2 ) {
    if(vme_write(address,buf,n_words,wsiz) < 0) 
      printf("Error writing to VME\n");
  }
  else {
    if(vme_read(address,buf,n_words,wsiz) < 0)
      printf("Error reading from VME\n");
    else {
      printf("***** Result from reading *****\n");
      i = 0;
      if(wsiz == SIXTEEN_BITS) n_words = n_words*2;
      while(i<n_words) {
	if(rw_opt == 0) s = 0xFF & (short)buf[i++];
	else {
	  s = (*((short *)(&buf[i]))) &0xffff;
	  i+=2;
	}
	printf(" Value: %hx\n",s);
      }
      if(wsiz == SIXTEEN_BITS) n_words = n_words/2;
    }
  }
}

vme_info()
{
  int i;
#ifdef BIT3
  printf("**** VME memory is divided as follows ****\n");
  printf(" 1. VMEbus i/o (address modifier 0x2d)\n");
  printf(" 2. VMEbus RAM (address modifier 0x0d)\n");
  printf("    address range 0x00000000 - 0x7fffffff\n");
  printf(" 3. VMEbus extended RAM (address modifier 0x0d)\n");
  printf("    address range 0x80000000 - 0xffffffff\n");
  printf(" 4. VMEbus 24 bits RAM (address modifier 0x3d)\n");
  printf("    address range 0x00000000 - 0x00ffffff\n");
  i = 0;
  printf(" Select which segment you want to use (1-4)\n");
  while((i<1) || (i>4)){
    printf("segment >>");
    scanf("%s",tmp);
    sscanf(tmp,"%d",&i);
  }
  if(i == 1) strcpy(devname,"/dev/bte64");
  if(i == 2) strcpy(devname,"/dev/bte96");
  if(i == 3) strcpy(devname,"/dev/bte128");
  if(i == 4) strcpy(devname,"/dev/bte160");
#endif
#ifdef DEC
  printf("**** VME memory is divided as follows ****\n");
  printf(" 1. VME A16/D16 space (0x8000 to 0xEFFF)\n");
  printf(" 2. VME A16/D32 space (0x4000 to 0x7FFF)\n");
  printf(" 3. VME A24/D16 space (0x800000 to 0xBFFFFF)\n");
  printf(" 4. VME A24/D32 space (0xC00000 to 0xFFFFFF)\n");
  printf(" 5. VME A32/D16 space (0x80000000 to 0x807FFFFF)\n");
  printf(" 6. VME A32/D32 space (0xC0000000 to 0xC07FFFFF)\n");
  printf(" Select which segment you want to use (1-6)\n");
  while((i<1) || (i>6)){
    printf("segment >>");
    scanf("%s",tmp);
    sscanf(tmp,"%d",&i);
  }
  devtype = devmodes[i-1];
  printf(" Give the addres (hex) from which memory mapping should start\n");
  i = 0;
  while(i == 0){
    printf("Mapping address  >> ");
    scanf("%s",tmp);
    if(sscanf(tmp,"%x",&start_addr) == 1) i = 1;
  }
  printf(" Give the number of bytes (int) to map\n");
  i = 0;
  while(i == 0){
    printf("Mapping size  >> ");
    scanf("%s",tmp);
    if(sscanf(tmp,"%d",&map_size) == 1) i = 1;
  }
#endif
}

vme_open()
{
#ifdef BIT3
  bt_status_t status_flags;
#endif

#ifdef BIT3
  if((i_dev = open(devname,O_RDWR))< 0) {
    printf("Device %s cannot be opened\n",devname);
    exit(0);
  }
  ioctl(i_dev,BIOC_SETUP,&status_flags);
  if(status_flags & BT_STATUS_MASK) {
    printf("Cannot initialize the BIT3 adapter\n");
    close(i_dev);
    exit(0);
  }
#endif
#ifdef DEC
  i_dev = (int)vme_mapopen(devtype, start_addr,map_size);
  if(i_dev == -1){
    printf("Device cannot be opened\n");
    exit(0);
  }
#endif
}

vme_read(addr,bf,nw,ws)
int addr;
short *bf;
int nw,ws;
{
  short *ts;
  int i;
#ifdef BIT3
  bt_param_t tmp;
#endif

#ifdef BIT3
  tmp.param = DATA_SIZE;
  if(ws == EIGHT_BITS) tmp.value = DATA8_SIZ;
  else                 tmp.value = DATA16_SIZ;
  ioctl(i_dev,BIOC_PARAM,&tmp);
  if(lseek(i_dev,addr,SEEK_SET) < 0) return(-1);
  if(ws == EIGHT_BITS) return(read(i_dev,bf,nw));
  else  return(read(i_dev,bf,2*nw));
#endif
#ifdef DEC
  if(ws == EIGHT_BITS) 
    memcpy((char *)bf,(char *)(i_dev+addr-start_addr),nw);
  else {
    ts = (short *)(i_dev+addr-start_addr);
    for(i=0;i<nw;i++){
      *bf++ = *ts++; 
    }
  }
#endif
}

vme_write(addr,bf,nw,ws)
int addr;
short *bf;
int nw,ws;
{
  short *ts;
  int i;
#ifdef BIT3
  bt_param_t tmp;
#endif

#ifdef BIT3
  tmp.param = DATA_SIZE;
  if(ws == EIGHT_BITS) tmp.value = DATA8_SIZ;
  else                 tmp.value = DATA16_SIZ;
  ioctl(i_dev,BIOC_PARAM,&tmp);
  if(lseek(i_dev,addr,SEEK_SET) < 0) return(-1);
  if(ws == EIGHT_BITS) return(write(i_dev,bf,nw));
  else  return(write(i_dev,bf,2*nw));
#endif
#ifdef DEC
  if(ws == EIGHT_BITS) 
    memcpy((char *)(i_dev+addr-start_addr),(char *)bf,nw);
  else {
    ts = (short *)(i_dev+addr-start_addr);
    for(i=0;i<nw;i++){
      *ts++ = *bf++; 
    }
  }
#endif
}

vme_close()
{
#ifdef BIT3
  close(i_dev);
#endif
}

