/*---------------------------------------------------------------*/
/*                                                               */
/*      Cengiz ERBAS                                             */
/*      Superconducting Super Collider Laboratory                */
/*      July 15, 1992                                            */
/*                                                               */
/*---------------------------------------------------------------*/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#include <stdio.h>
#include <signal.h>
#include "from_pipe.h"
#include "template.h"
#include <sys/time.h>


int Process_Event();
int Check_Event_Usr();


/*
 *   For NOVA
 */
#include <string.h>
#include "nova.h"

#define NOVA_HOSTNAME_MAX  32    /*  extracted from novanet.h   */
#define DEFAULT_NAME          "/tmp/nova"
#define DEFAULT_SIGNIFICANCE  70
#define PRI_COLLECTOR         -1 

#define BL_LEN 300

#define  NTIMEOUT       0

#define NOVA_FLUSH      0
#define NOVA_GET 	1
#define NOVA_PUT 	2

struct   NOVAD *nd;

int event_semid, fake_sem; 
int run_number;

void Event_Sig_Handler_Usr();
void Process_Idle();
extern char *get_name_from_args();
extern long send_err_mess();

struct timeval tp;
struct timezone tzp;
extern int eventnr;
int  initial_sec, start_time;

int  max_length;



/*(a+out) ------------------------------------------------- */
void main (argc, argv)
int argc;
char *argv[];
/*(b+out) ------------------------------------------------- 
Procedure Name
Main - main procedure of the 
Synopsis
Description
main(argc, argv) is the main procedure of the              

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */	

{
   char name[NV_MAXNAME + 1];
   char sec_name[NV_MAXNAME + NOVA_HOSTNAME_MAX + 1];
   user_init();
   Create_Sem(&event_semid);

   if(Template_Init(get_name_from_args(argc,argv)) < 0){
     printf("Process of name %s already exists\n",
            get_name_from_args(argc,argv));
     printf("Please use -n to set a different name\n");
     exit(0);
   }

   /*
    *   Open NOVA
    */
   UNI_nova_parms(DEFAULT_NAME, PRI_COLLECTOR, name, sec_name);
   if ((int)(nd = (struct NOVAD *)UNI_nova_open(name, PRI_COLLECTOR, 1)) 
                  == -1) {
       if (debug_level >= 10) printf("from_pipe: cannot open %s", name);
       Send_Message ( ownpname, "EXIT" );
   }

   Template_Main((int (*)()) Process_Event, NULL, 
                 (void (*)()) Process_Idle); 
}



int Process_Event()
{
  int *data;
  int event_length;
   char msg[100];
static int timedout=0;
Table entry;


  while (get_buf(&data, max_length) !=0) ;
   status =  read_event(data, &event_length);
   if(status == 2) { /* need longer max_length */
   	   max_length = event_length;
   	   flush_buf();
   	   
   }
   	   
   else if( status == 1) {
   /* end of file */
      flush_buf();
        Send_Message ( ownpname, "EXIT" );
   } 

  if (put_buf(  max_length, event_length, 70) != 0) {
     while (get_buf( &data, max_length) !=0) {
    	/*    printf(" get_buf failure (b)\n"); */
     }
  }
  return (TRUE);

    
}

void Process_Idle()
{

}



int ExitR (header, arg_string)
char *header, *arg_string;
{
   if (Delete_Sem(event_semid) == ERROR)
      if (debug_level >= 10) printf("Error in Delete_Sem()\n");
   UNI_nova_close(nd);
   return(Exit("from_pipe", " "));
}


user_init()
 /* masx length in 4 byte words for event */
{ max_length=200; /* modify here to return different max_length */

return 0;
}

int read_event(data,event_length)
int *data;
int *event_length;
{
  static int block_buf[BL_LEN];
  static int first=1;
	static int bp;
	static int status;
	int pos;
	
  if( first ==1) { first=0;  
         status = read_block(block_buf);
         if(status == 0) { /* no bytes read */
             first = 1; 
         		 return (1); /* EOF  */
         }		
  		   bp =0;
  }
  
  
  pos=0;
  if (bp >= status) {
         status = read_block(block_buf) ;
         bp =0;
  }
  if(status == 0) { /* no bytes read */
             first=1;
         		 return (1) ;/* EOF  */
  }	
  
  	
  *event_length = block_buf[bp];
  if(*event_length > max_length) {
  		printf(" max_length too small in read_event \n");
  		/* the calling program should call flush_buf 
  			  and then get_buf with a new larger value of max_length 
  		*/
  			  
  		return (2);
  		}
  pos=0;
  while(pos < *event_length || *event_length == 0 ) {		
     if (bp >= status) {
         status = read_block(block_buf) ;
         bp =0;
     }
         if(status == 0) { /* no bytes read */
             first=1;
         		 return (1) ;/* EOF  */
         }		 
         if (print_level >= 30) printf(" %d %d\n",bp,block_buf[bp]);

			   if(*event_length != 0) {
  	     	data[pos]=block_buf[bp];
  	      pos++; 
  	     }
  	    else { 
  	       if ( print_level >= 10) 
  	          printf(" event_length = 0 at bp=%d\n",bp);
  	     } 
  	     bp++;
  }
return (0);
}

int read_block(block_buf)
int *block_buf;
{ 
  int status;

  if (print_level >= 10) printf("Inside READ_BLOCK\n");
    	
  status =fread( (char *)block_buf, sizeof(int),BL_LEN, stdin) ;
  return (status);  

}
