/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/
/*********************************************************
 uni_action.c: the actions to be performed when clicking buttons.
 Charles Timmermans, University of Michigan
 *********************************************************/
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#ifdef HP
#include <symlink.h>
#endif

struct cnf_type {
  char txt[100];
  int (*action)();
};

extern char editor[];
extern char gen_txt[];
extern struct cnf_type bt[];
extern int n_but;
extern int create_start();
extern int check_disp2();

int config_nodefile()
{
/**********************************************
 edits the nodefile with your favorite editor.
 Afterwards, it reads the nodefile and checks if 
 Start_node files are there. If not a minimal
 Start script is created
 ***********************************************/
  char tmp[200],nfil[100],bdir[100],nde[100],flavor[100],*c;
  FILE *fp,*fps;

  sprintf(nfil,"%s/conf/nodefile",getenv("USERUNIDAQ"));
  replace_link(nfil);
  sprintf(tmp,"%s %s",editor,nfil);
  system(tmp);
/*SSC  sprintf(bdir,"%s/bin",getenv("USERUNIDAQ")); */
  sprintf(bdir,"%s/conf",getenv("USERUNIDAQ"));
  if((fp = fopen(nfil,"r")) == NULL) return(-1);
  while(fscanf(fp,"%s",nde) == 1) {
    fscanf(fp,"%s",flavor);
    sprintf(tmp,"%s/Start_%s",bdir,nde);
    if((fps = fopen(tmp,"r")) != NULL) {
      fclose(fps);
    }
    else {
      fps = fopen(tmp,"w");
      if (strncasecmp(flavor,"UNIX",4) == 0)  {                   /* UNIX */
        fprintf(fps,"#!/bin/csh\n");
        fprintf(fps,"onintr -\n");
        fprintf(fps,"source $USERUNIDAQ/setup.csh\n");
        fprintf(fps,"msg_server >& /dev/null &\n");
        fprintf(fps,"xpc >& /dev/null &\n");
      }
      else  {                                     /* VXWORKS */
	fprintf(fps,"cd \"%s/conf\"\n",getenv("USERUNIDAQ"));
	fprintf(fps,"ld <vxshm.o\n");
	fprintf(fps,"cd \"%s/NOVA/src\"\n",getenv("TOPUNIDAQ"));
	fprintf(fps,"ld <vxnovad\n");
	fprintf(fps,"cd \"../netsrc\"\n");
	fprintf(fps,"ld <vxnovand\n");
	fprintf(fps,"cd \"../../bin\"\n");
	fprintf(fps,"sp novad_main, \"dummy\"\n");
	fprintf(fps,"sp novand_main\n");
	fprintf(fps,"ld <msg_server\n");
	fprintf(fps,"sp msg_server\n");
	fprintf(fps,"ld <status\n");
	fprintf(fps,"ld <repair\n");
	fprintf(fps,"ld <ask\n");
      }
      fclose(fps);
      sprintf(nfil,"chmod 775 %s",tmp);
      system(nfil);
    }
  }
  fclose(fp);
  return(0);
}

int config_exit()
/********************************************
 exits the configuration tool
 *******************************************/
{
  exit(0);
}

edit_start(but)
int but;
/**********************************************************
 edits a Start-script corresponding to the text on a button.
 This uses the second display, not the first.
 **********************************************************/
{
  char stfil[100],tmp[300];
  
/*SSC  sprintf(stfil,"%s/bin/%s",getenv("USERUNIDAQ"),bt[but].txt); */
  sprintf(stfil,"%s/conf/%s",getenv("USERUNIDAQ"),bt[but].txt);
  replace_link(stfil);
  sprintf(tmp,"%s %s",editor, stfil);
  system(tmp);
  sprintf(tmp,"rm -f %s~ %s.bak %s.BAK %s%%",stfil,stfil,stfil,stfil);
  system(tmp);
}

int config_start()
/****************************************************
 defines a second display, which contains buttons that
 when clicked upon causes editing Start-scripts
 ***************************************************/
{
  FILE *fp;
  char ndfil[100],tmp[100],flavor[100];
  
  sprintf(ndfil,"%s/conf/nodefile",getenv("USERUNIDAQ"));
  n_but = 0;
  if((fp = fopen(ndfil,"r")) == NULL) return(-1);
  while(fscanf(fp,"%s",tmp) == 1) {
    sprintf(bt[n_but].txt,"Start_%s",tmp);
    bt[n_but++].action = edit_start;
    fscanf(fp,"%s",flavor);
  }
  fclose(fp);
  strcpy(gen_txt,"Start_files");

  check_disp2();
  return(0);
}

edit_runco(but)
int but;
/**********************************************************
 edit runcontrol files corresponding to the text on a button.
 Again it uses a second display. When the text is "make", there
 is no file edited, but runcontrol is remade.
 ***********************************************************/
{
  char stfil[100],tmp[300];
  
  if(strcmp(bt[but].txt,"make") == 0){
    sprintf(tmp,"cd %s/src/control/runcontrol; make clean; make",
	    getenv("TOPUNIDAQ")); 
    system(tmp);
    return;
  }
  if(strcmp(bt[but].txt,"command.list") == 0)
    sprintf(stfil,"%s/conf/command.list",
	    getenv("USERUNIDAQ"));
  else
    sprintf(stfil,"%s/src/control/runcontrol/%s",
	    getenv("TOPUNIDAQ"),bt[but].txt);
  replace_link(stfil);
  sprintf(tmp,"%s %s",editor, stfil);
  system(tmp);
}

int config_command()
/************************************************
 configure a second window, that eases runcontrol
 updates
 ************************************************/
{

  n_but = 0;
  sprintf(bt[n_but].txt,"command.list");
  bt[n_but++].action = edit_runco;
  sprintf(bt[n_but].txt,"commands.h");
  bt[n_but++].action = edit_runco;
  sprintf(bt[n_but].txt,"commands_lib.c");
  bt[n_but++].action = edit_runco;
  sprintf(bt[n_but].txt,"make");
  bt[n_but++].action = edit_runco;
  strcpy(gen_txt,"Runcontrol configuration");
  check_disp2();
  return(0);
}

edit_operator(but)
int but;
/**********************************************************
 edit operator files corresponding to the text on a button.
 Again it uses a second display. When the text is "make", there
 is no file edited, but operator is remade.
 ***********************************************************/
{
  char stfil[100],tmp[300];
  
  if(strcmp(bt[but].txt,"make") == 0){
    sprintf(tmp,"cd %s/src/control/operator; make clean; make",
	    getenv("TOPUNIDAQ")); 
    system(tmp);
    return;
  }
  if(strcmp(bt[but].txt,"runpar display") == 0)
    sprintf(stfil,"%s/src/control/operator/runpar.h",getenv("TOPUNIDAQ"));
  else if(strcmp(bt[but].txt,"conversion") == 0)
    sprintf(stfil,"%s/src/control/operator/runp_conv.c",getenv("TOPUNIDAQ"));
  else if(strcmp(bt[but].txt,"runco buttons") == 0)
    sprintf(stfil,"%s/src/control/operator/runcon.h",getenv("TOPUNIDAQ"));
  else if(strcmp(bt[but].txt,"button actions") == 0)
    sprintf(stfil,"%s/src/control/operator/runcon_act.c",getenv("TOPUNIDAQ"));
  else return;
  replace_link(stfil);
  sprintf(tmp,"%s %s",editor, stfil);
  system(tmp);
}

int config_operator()
/************************************************
 configure a second window, that eases operator
 updates
 ************************************************/
{
  char tmp[100];

  n_but = 0;
  sprintf(bt[n_but].txt,"runpar display");
  bt[n_but++].action = edit_operator;
  sprintf(bt[n_but].txt,"conversion");
  bt[n_but++].action = edit_operator;
  sprintf(bt[n_but].txt,"runco buttons");
  bt[n_but++].action = edit_operator;
  sprintf(bt[n_but].txt,"button actions");
  bt[n_but++].action = edit_operator;
  sprintf(bt[n_but].txt,"make");
  bt[n_but++].action = edit_operator;
  strcpy(gen_txt,"Operator configuration");
  check_disp2();
  return(0);
}

edit_collector(but)
int but;
/**********************************************************
 edit collector files corresponding to the text on a button.
 Again it uses a second display. When the text is "make", there
 is no file edited, but collector is remade.
 ***********************************************************/
{
  char stfil[100],tmp[300];
  
  if(strcmp(bt[but].txt,"make") == 0){
    sprintf(tmp,"cd %s/collector; make clean; make",
	    getenv("USERUNIDAQ")); 
    system(tmp);
    return;
  }
  if(strcmp(bt[but].txt,"User code") == 0)
    sprintf(stfil,"%s/collector/user_event.c",getenv("USERUNIDAQ"));
  else if(strcmp(bt[but].txt,"Camac code") == 0)
    sprintf(stfil,"%s/collector/camac_event.c",getenv("USERUNIDAQ"));
  else return;
  replace_link(stfil);
  sprintf(tmp,"%s %s",editor, stfil);
  system(tmp);
}

int config_collector()
/************************************************
 configure a second window, that eases collector
 updates
 ************************************************/
{
  char tmp[100];

  n_but = 0;
  sprintf(bt[n_but].txt,"User code");
  bt[n_but++].action = edit_collector;
  sprintf(bt[n_but].txt,"Camac code");
  bt[n_but++].action = edit_collector;
  sprintf(bt[n_but].txt,"make");
  bt[n_but++].action = edit_collector;
  strcpy(gen_txt,"Collector configuration");
  check_disp2();
  return(0);
}

edit_dataview(but)
int but;
/**********************************************************
 edit collector files corresponding to the text on a button.
 Again it uses a second display. When the text is "make", there
 is no file edited, but collector is remade.
 ***********************************************************/
{
  char stfil[100],tmp[300];
  
  if(strcmp(bt[but].txt,"make") == 0){
    sprintf(tmp,"cd %s/dataview; make clean; make",
	    getenv("USERUNIDAQ")); 
    system(tmp);
    return;
  }
  sprintf(stfil,"%s/dataview/%s",
	  getenv("USERUNIDAQ"),bt[but].txt);
  replace_link(stfil);
  sprintf(tmp,"%s %s",editor, stfil);
  system(tmp);
}

int config_dataview()
/************************************************
 configure a second window, that eases dataview
 updates
 ************************************************/
{
  char tmp[100];

  n_but = 0;
  sprintf(bt[n_but].txt,"datawin.h");
  bt[n_but++].action = edit_dataview;
  sprintf(bt[n_but].txt,"dataconv.c");
  bt[n_but++].action = edit_dataview;
  sprintf(bt[n_but].txt,"make");
  bt[n_but++].action = edit_dataview;
  strcpy(gen_txt,"Dataview configuration");
  check_disp2();
  return(0);
}

replace_link(lnk)
char *lnk;
/*******************************************
 When the directory entry is a link, the link is
 replaced by the file it links to
 *******************************************/
{

  char f1[100];
  char bf[200];
  int i;

  i = (int) readlink(lnk,f1,100);
  if(i <=0) return;
  unlink(lnk);
  sprintf(bf,"cp %s %s",f1,lnk);
  system(bf);
}
