/*---------------------------------------------------------------*/
/*                                                               */
/*      Cengiz ERBAS                                             */
/*      Superconducting Super Collider Laboratory                */
/*           DECstation analyzer -- 4/2/93                       */
/*                                                               */
/*---------------------------------------------------------------*/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#include <stdio.h>
#include <sys/time.h>
#include <signal.h>
#include <string.h>
#include "analyzer.h"
#include "template.h" 

int Process_Event();

struct procedures *searchproc(); 

#ifdef Linux
#define ana_exec_ ana_exec__
#define ana_init_ ana_init__
#define ana_beg_ ana_beg__
#define ana_end_ ana_end__
#define ana_exit_ ana_exit__
#endif

extern ana_exec_();
extern ana_init_();
extern ana_beg_();
extern ana_end_();
extern ana_exit_();
 
/*
 *   For NOVA
 */
#include "nova.h"
#define NOVA_HOSTNAME_MAX  32    /*  extracted from novanet.h   */

#define DEFAULT_NAME     "/tmp/nova"
#define DEFAULT_PRI      20
#define DEFAULT_SLEEP    1

#define  NTIMEOUT        0

void interrupt_handler();

struct   NOVAD *nd;

int run_number = -1;


main(argc, argv)
int argc;
char *argv[];
{
  int proc_ok = 0;
  char name[NV_MAXNAME + 1];
  char sec_name[NV_MAXNAME + NOVA_HOSTNAME_MAX + 1];
   
  if(Template_Init(get_name_from_args(argc,argv)) < 0){
    printf("Process of name %s already exists\n",
           get_name_from_args(argc,argv));
    printf("Please use -n to set a different name\n");
    exit(0);
  }

  UNI_nova_parms(DEFAULT_NAME, DEFAULT_PRI, name, sec_name);
  if ((int)(nd = (struct NOVAD *)UNI_nova_open(name, nova_prio, 1)) 
               == -1) {
      printf("Error in opening nova...\n");
      Send_Message ( ownpname, "EXIT" );
      proc_ok = 1;
  }

  if (proc_ok == 0) ana_init_();

  Template_Main((int (*)()) Process_Event, NULL, NULL);
}


/*(a+out) ------------------------------------------------- */
int Process_Event()
/*(b+out) -------------------------------------------------
Procedure Name
Process_Event - processes an event
Synopsis
Description
Process_Event() processes an event.       

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{
  struct timeval tp;
  struct timezone tzp;
  char   macro_cmd[80], num[20];
  int    status, pos;
  int 	*data;
  int length;
  int i;
  
  /* 
   *   Check for event
   */
  /* nova get : keep trying to get a buffer */

  status = UNI_nova_get(nd, &data, &length);
  if (status != 0) return(FALSE);
  pos = 0;
  while (pos*4 < length) {
    if (data[pos] == 0) break;
    if (debug_level > 10) {
      if (data[pos + 1] == EVENT_RECORD)
	if (debug_level > 20) printf("Event buffer received.\n");
	else if (data[pos + 1] == BEGINRUN_RECORD)
	  printf("BeginRun record received.\n");
	else if (data[pos + 1] == PAUSERUN_RECORD)
	  printf("PauseRun record received.\n");
	else if (data[pos + 1] == RESUMERUN_RECORD)
	  printf("ResumeRun record received.\n");
	else if (data[pos + 1] == ENDRUN_RECORD)
	  printf("EndRun record received.\n");
    }

    if (data[pos + 1] == BEGINRUN_RECORD) {
      sprintf(num,"%d", data[pos+2]);
      Begin(ownpname, num);
      pos += data[pos];
    }
    else {
      if (data[pos + 2] != run_number) {
	sprintf(num,"%d", data[pos+2]);
	printf(" Warning: unexpected change of runnumber: ");
	printf(" %d --> %d\n", run_number, data[pos+2]);
	Begin("recorder", num);
      }
      if (data[pos + 1] == ENDRUN_RECORD) {
	End();
      }
      else if (data[pos + 1] == EVENT_RECORD) {
	ana_exec_(&data[pos+6],&data[pos]);
      }
      pos += data[pos];
    }
  }
  
  UNI_nova_put(nd, data, length, 0);
  return(TRUE);
}


/*(a+out) ------------------------------------------------- */
int Begin(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
Begin - processes a begin command
Synopsis
Description
Begin(header, arg_string) processes a begin command.  It also 
execute the andbegin.kumac macro. 
header: the name of the process which sends the begin cmnd,  
arg_string: the arguments of the begin command. 

Superconduction Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{
  char command[100];

  run_number = atoi(arg_string);
  runstate = BEGIN_STATE;
  printf("Begin Run # %d \n",run_number);
  ana_beg_();

  return TRUE;
}



/*(a+out) ------------------------------------------------- */
int End(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
End - Main procedure of histogram
Synopsis
Description
End(header, arg_string) processes an end command. It also
executes the anend.kumac macro.           
header: the name of the process which sends the end command, 
arg_string: the arguments of the end command.  	 		     

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */   

{
  if (debug_level >= 20) printf("End()...\n");
  runstate = END_STATE;
  ana_end_( );
  return TRUE;
}

/*(a+out) ------------------------------------------------- */
int ExitA(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
ExitA - processes a display command
Synopsis
Description
Display(header, arg_string) processes a display command.     
header: the name of the process which sends the display cmnd,
arg_string: the arguments of the display command.

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */ 

{
   UNI_nova_close(nd);
   kill(getppid(), SIGINT);
   Exit(ownpname, " ");
}

