/********************************************************************
 File dataconv.c: contains the conversion routines from data to 
                  string
 author: C. Timmermans.
 ********************************************************************/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/
#include "template.h"


/*****************************************************************
 The conversion routines have the following form:

  routine(header,data,txt)
  int *header,*data;
  char *txt;
  {
  *************************************************************
  Using the header and the data-buffer, one has to fill the txt
  character string. As an example I will fill txt with the 
  hexadecimal rep. of the first dataword, but only if this 
  data corresponds to an actual event
  **************************************************************
  if(header[1] == EVENT_RECORD) sprintf(txt,"%x",data[0]);
  else strcpy(txt,"No Dataword");
  }
******************************************************************/
adc_volt_5(header,data,txt)
int *header,*data;
char *txt;
{
  adc_volt(header,data,5,txt);
}

adc_volt_10(header,data,txt)
int *header,*data;
char *txt;
{
  adc_volt(header,data,10,txt);
}

adc_volt_15(header,data,txt)
int *header,*data;
char *txt;
{
  adc_volt(header,data,15,txt);
}

adc_volt_20(header,data,txt)
int *header,*data;
char *txt;
{
  adc_volt(header,data,20,txt);
}

adc_volt_25(header,data,txt)
int *header,*data;
char *txt;
{
  adc_volt(header,data,25,txt);
}

adc_volt_30(header,data,txt)
int *header,*data;
char *txt;
{
  adc_volt(header,data,30,txt);
}

adc_volt(header,data,chan,txt)
int *header,*data,chan;
char *txt;
{
  int vlt,stat;
 
  if(header[1] == BEGINRUN_RECORD) strcpy(txt,"Begin");
  else if(header[1] == PAUSERUN_RECORD) strcpy(txt,"Pause");
  else if(header[1] == ENDRUN_RECORD) strcpy(txt,"End");
  else if(header[1] == RESUMERUN_RECORD) strcpy(txt,"Resume");
  else if(header[1] == EVENT_RECORD) {
    vlt = data[2*chan];
    stat = data[2*chan+1];
    if(stat != 0) strcpy(txt,"Status Error");
    else sprintf(txt,"%f V",.002412*((float)vlt-2061.));
  }
  else strcpy(txt,"Unknown runstate");
}

