/*---------------------------------------------------------------*/
/*                                                               */
/*      recorder.c                                               */
/*      Donald BRENNER                                           */
/*      Superconducting Super Collider Laboratory                */
/*      July 20, 1992                                            */
/*                                                               */
/*---------------------------------------------------------------*/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#ifdef VXWORKS
#include <vxWorks.h>
#include <stdioLib.h>
#include <msgQLib.h>
#include <scsiLib.h>
#include "exabyte.h"
#else
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/mtio.h>
#include <stdio.h>
#include<fcntl.h>
#endif
#ifdef SGI 
#define _BSD_SIGNALS 
#endif 
#include <signal.h>
#include "recorder.h"
#include "template.h"

#ifdef SOLARIS2
struct  sigvec {
#if defined(__cplusplus)
	void	(*sv_handler)(int, int, struct sigcontext *, char *);
#else
	void	(*sv_handler)();	/* signal handler */
#endif
	int	sv_mask;		/* signal mask to apply */
	int	sv_flags;		/* see signal options below */
};
#endif

#include "nova.h"
#define NOVA_HOSTNAME_MAX  32    /*  extracted from novanet.h   */
#define DEFAULT_NAME     "/tmp/nova"
#define DEFAULT_PRI      80
#define DEFAULT_SLEEP    1

#define  NTIMEOUT       0
#define  LRECL          60000
char tapestor[LRECL];  /* a buffer to store data before writing to tape 
                          records will never contain more than 1  NOVA 
                          buffers */
int  tapeptr = 0;     /* where we are in the tape buffer */

int Process_Event();
void Process_Idle();
void openfile();
void opentape();
void addzeros();
extern char *get_name_from_args();
#ifndef VXWORKS
extern long send_err_mess();
void SIGSEGV_handler();
#endif
struct   NOVAD *nd;
int *data, length;

FILE *fp = NULL;
void wait_for_device();
int scsi_id = 0;
#ifdef VXWORKS
/* A max. of 20 tape drives */
/* Is scsi initialized ?? */
short iniscsi = 0;
SCSI_PHYS_DEV *tape_dev[20] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
#else
struct mtop mt_command;
int tape_dev[20] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
char tapenam[80][20];
#endif

int run_number  = -1;


/*(a+out) ------------------------------------------------- */
#ifdef VXWORKS
recorder(argv1,argv2,argv3,argv4,argv5,argv6,argv7,
          argv8,argv9,argv10) 
char *argv1,*argv2,*argv3,*argv4,*argv5,*argv6,*argv7,*argv8,*argv9,*argv10;
{
  char *argv[] = {NULL,argv1,argv2,argv3,argv4,argv5,argv6,argv7,argv8,
		    argv9,argv10}; 
  int argc = 11;
#else
void main (argc, argv)
int argc;
char *argv[];
{
#endif
/*(b+out) -------------------------------------------------
Procedure Name
Main - main procedure of recorder
Synopsis
Description
main(argc, argv) is the main procedure of the recorder. It
calls the template with the addresses of the Process_Event()
procedures. It also calls the Template_Init.                           

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

   char name[NV_MAXNAME + 1];
   char sec_name[NV_MAXNAME + NOVA_HOSTNAME_MAX + 1];
   struct sigvec vec;
   short s;
   

#ifdef VXWORKS
   argv[0] = "recorder";
#endif
   strcpy(dest_dir, "");
   strcpy(eventsink, "");

   if(Template_Init(get_name_from_args(argc,argv)) < 0){
     printf("Process of name %s already exists\n",
            get_name_from_args(argc,argv));
     printf("Please use -n to set a different name\n");
     exit(0);
   }

   UNI_nova_parms(DEFAULT_NAME, DEFAULT_PRI, name, sec_name);
   if ((int)(nd = (struct NOVAD *) UNI_nova_open(name, nova_prio, 1)) 
                == -1) {
       printf("Error in opening nova...\n");
       Send_Message ( ownpname, "EXIT" );
   }
#ifndef VXWORKS
#ifdef SGI
   vec.sv_handler = (int (*)()) SIGSEGV_handler;
#else
   vec.sv_handler = (void (*)()) SIGSEGV_handler;
#endif
   vec.sv_mask = 0;
   vec.sv_flags = 0;
#ifdef HP
   sigvector(SIGKILL, &vec, (struct sigvec *) NULL);
#else
   sigvec(SIGKILL, &vec, (struct sigvec *) NULL); 
#endif
#endif
   Crit_Var("eventsink");
   read_value_from_sp(ownpname,"eventsink",&s,event_sink);
   Crit_Var("dest_dir");
   read_value_from_sp(ownpname,"dest_dir",&s,destdir);
   Crit_Var("device");
   read_value_from_sp(ownpname,"device",&s,devname);

   Template_Main((int (*)())Process_Event, NULL, (void (*)())Process_Idle);
}





/*(a+out) ------------------------------------------------- */
int Process_Event()
/*(b+out) -------------------------------------------------
Procedure Name
Process_Event - receives the buffer and writes the data to the file
Synopsis
Description
Process_Event() receives the buffer and if the buffer is an event
of recorder, begin, resume or end type it write the data to the
file.  Calls fwrite, nova_get, fclose, strcasecmp and begin.  

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
  int status;
  char num[20];
  int pos = 0;
  
  num[0] = '\0';
/* Update all variables when needed */
  if(runstate == END_STATE ) {
    strcpy(eventsink,event_sink);
    strcpy(dest_dir,destdir);
    set_value_in_sp("eventsink",event_sink);
    set_value_in_sp("dest_dir",destdir);
    run_number = -1; /* Force the recorder to open a new file by changing 
                        the run number. */ 
  }
  else {
/*
 *         Restore global variables from the local copies.  
 */
    strcpy(event_sink,eventsink);
    strcpy(destdir,dest_dir);
  }

/* Get the NOVA buffer */
  status = UNI_nova_get(nd, &data, &length);
  if (status != 0) return (FALSE); 
  

/** Loop over events **/  
  tapeptr = 0;
  while (pos*4 < length && pos*4 >= 0 ) {
    if (data[pos] == 0) break;
/*
 * Event driven commands 
 */
    if ((data[pos+1] == BEGINRUN_RECORD) || (data[pos+2] != run_number)) {
      run_number = data[pos+2];
      Begin(ownpname,num);
    }
/* Write data to file/tape or not at all */
    write_output(pos,0);
    if (data[pos+1] == ENDRUN_RECORD) End(ownpname,num);
    pos+=data[pos];
  }
/* release NOVA buffer */
  write_output(0,1);
  UNI_nova_put(nd, data, length, 0); 
  return (TRUE);
}

write_output(pos,flush)
int pos,flush;
{
  int len,dtptr;
  
  if(flush == 1) len = LRECL;
  else len=4*data[pos];
  if (strcasecmp(eventsink, "dummy") == 0) return(0);
  else if (strcasecmp(eventsink, "tape") == 0) {
    if(tape_dev[scsi_id] == 0 || len < 0 ) return(-1);
    if(flush == 1) {
#ifdef VXWORKS
      exbWrite(tape_dev[scsi_id],tapestor, &tapeptr);
#else
      write(tape_dev[scsi_id],tapestor,tapeptr);
#endif
      tapeptr = 0;
      return(0);
    }
    if((tapeptr+len) < LRECL){
      bcopy((char *)((int)(data)+4*pos),tapestor+tapeptr,len);
      tapeptr+=len;
    }
    else {
      dtptr = 0;
      while((tapeptr+len-dtptr) >= LRECL) {
	bcopy((char *)((int)(data)+4*pos+dtptr),tapestor+tapeptr,
	      LRECL-tapeptr);
	dtptr += (LRECL-tapeptr);
	tapeptr = LRECL;
#ifdef VXWORKS
	exbWrite(tape_dev[scsi_id],tapestor, &tapeptr);
#else
	write(tape_dev[scsi_id],tapestor,tapeptr);
#endif
	tapeptr = 0;
      }
      bcopy((char *)((int)(data)+4*pos+dtptr),tapestor,len-dtptr);
      tapeptr=len-dtptr;
    }
  }
/* Default: DISK */
  else {
    if(flush == 1) return(0);
    if ( (len>0) && (fp != NULL) ) {
      fwrite(data+pos, 4, data[pos], fp);
    }
  }
}



/*(a+out) ------------------------------------------------- */
int Begin(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
Begin - processes a begin command
Synopsis
Description
Begin(header, arg_string) processes a begin command.  It calls
openfile to open a file for recordering the incoming data.
header: the name of the process which sends the begin cmnd,
arg_string: the arguments of the begin command. 
                                 
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   char temp1[100], temp2[100];
	 	 
   runstate = BEGIN_STATE;
   printf("Beginning..., trying to open a file\n");
   Parse(arg_string,temp1,temp2);
   if(strlen(temp1) > 0) {
     run_number = atoi(temp1);
   }
   strcpy(eventsink,event_sink);
   strcpy(dest_dir,destdir);
   set_value_in_sp("eventsink",event_sink);
   set_value_in_sp("dest_dir",destdir);

   if(strcasecmp(eventsink,"tape") == 0)          opentape();
   else if (strcasecmp(eventsink,"dummy") != 0)   openfile(run_number);

   return TRUE;
}



/*(a+out) ------------------------------------------------- */
int End(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
End - processes an end command
Synopsis
Description
End(header, arg_string) processes an end command.  Is responsible 
for closing the file.
header: the name of the process which sends the end command.
arg_string: the arguments of the end command.

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   runstate = END_STATE;
/* Flush output */
   write_output(0,1);
   if(strcasecmp(eventsink,"tape") == 0) {
#ifdef VXWORKS
     exbWriteFilemark(tape_dev[scsi_id]);
#else
     mt_command.mt_op = MTWEOF;
     mt_command.mt_count = 1;
     ioctl(tape_dev[scsi_id],MTIOCTOP,&mt_command);
#endif
   }
   else {
     fclose(fp);
     fp = NULL;
#ifndef VXWORKS
     send_err_mess(0,"Recorder closes data file at End Run");
#endif
   }
   return TRUE;
}


/*(a+out) ------------------------------------------------- */
int ExitR(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
Exit - processes an end command
Synopsis
Description
End(header, arg_string) processes an exit command.  Calls end to
close the file and then calls nova_close.
header: the name of the process which sends the end command.
arg_string: the arguments of the end command.

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   UNI_nova_close(nd);
   Exit(ownpname, " ");
}


int scsi_load()
{
  opentape();
#ifdef VXWORKS
  exbLoadTape(tape_dev[scsi_id]);
  wait_for_device();
#else
  mt_command.mt_op = MTNOP;
  mt_command.mt_count = 1;
  ioctl(tape_dev[scsi_id],MTIOCTOP,&mt_command);
#endif
  return TRUE;
}

int scsi_unload()
{
  if(tape_dev[scsi_id] != 0){
#ifdef VXWORKS
    exbUnloadTape(tape_dev[scsi_id]);
#else
    mt_command.mt_op = MTOFFL;
    mt_command.mt_count = 1;
    ioctl(tape_dev[scsi_id],MTIOCTOP,&mt_command);
#endif
  }
  return TRUE;
}

int scsi_rewind()
{
  if(tape_dev[scsi_id] != 0){
#ifdef VXWORKS
    exbRewind(tape_dev[scsi_id]);
#else
    mt_command.mt_op = MTREW;
    mt_command.mt_count = 1;
    ioctl(tape_dev[scsi_id],MTIOCTOP,&mt_command);
#endif
  }
  return TRUE;
}

int scsi_end()
{
  if(tape_dev[scsi_id] != 0){
#ifdef VXWORKS
    exbWriteFilemark(tape_dev[scsi_id]);
    exbWriteFilemark(tape_dev[scsi_id]);
#else
    mt_command.mt_op = MTWEOF;
    mt_command.mt_count = 2;
    ioctl(tape_dev[scsi_id],MTIOCTOP,&mt_command);
#endif
  }
  return TRUE;
}


void wait_for_device()
{
#ifdef VXWORKS
  EXB_STATUS 	device_status;
  int error_count = 0;

  taskDelay (sysClkRateGet());     /* delay 1 second */
  device_status = exbTestReady (tape_dev[scsi_id]);
  while   ((device_status.errorCode == EXB_UNIT_ATTENTION)
	   || (device_status.errorCode == EXB_NOT_READY)
	   || (device_status.errorCode == EXB_BUSY))  {
    error_count += 1;
    if (error_count >= 300)
      {
	return ;
      }
    device_status = exbTestReady (tape_dev[scsi_id]);
  } /* while */
#endif
}



/*(a+out) ------------------------------------------------- */
void openfile(new_num)
int new_num;
/*(b+out) -------------------------------------------------
Procedure Name
openfile - opens a file for writing
Synopsis
Description
openfile(new_num) opens a new file with .dat extentions based
on the run number passed in by the buffer.  It calls addzores.  

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
    char name[100], num[30], the[100], pname[100];
    int i;
/*  int j=0; */
    char digit[10];
      
    run_number = new_num;
    if ( fp != NULL) {
       fclose(fp);
       if (debug_level >= 20) printf("Close a file...\n");
       fp = NULL;
#ifndef VXWORKS
       sprintf(the,"Recorder force-closes a file ");
       send_err_mess(0,the);
#endif
    }
/*
 * Pre-pend (or not) the destination directory to the manufactured
 * file name.
 */
    if (strcmp(dest_dir,"") == 0) {
       strcpy(name,"r");
    }
    else { 
       strcpy(name, dest_dir);
       strcat(name, "/");
       strcat(name, "r");
    }
    sprintf(num,"%7.7d",new_num);
    strcat(name, num);
    strcat(name, ".dat");
    strcpy(pname, name);
         
/*
 * Run number zero is a special case, always open it for writing.
 */
    if (run_number == 0) {
       fp = fopen(name,"w+b");
    }
/*
 * Find an unused file name to open.  Post-pend incrementing digits until
 * such a file is found.
 */
    else {
       fp = fopen(name, "r+b");
       if (fp == NULL) {                /*  Unused file name.  */
/*          fclose(fp);
*/
       	  fp = fopen(name, "w+b");
       }
       else {                           /*  Used -- find one unused  */
          i = 0;
          do {
             fclose(fp);
             fp = NULL;
             i++;
             sprintf(digit,"%d",i);
             strcpy(name,pname);
             strcat(name, digit);
             fp = fopen(name, "r+b");
          } while(fp != NULL);
/*          fclose(fp);
*/
          fp = NULL;
          fp = fopen(name, "w+b");
	}
      }
      if(((int)fp) != 0) {
        printf("Recorder opens file %s\n",name);
        sprintf(the,"Recorder opens file %s",name);
      }
      else {
        sprintf(the,"Recorder FAILS to open file %s",name);
      }
#ifndef VXWORKS
      send_err_mess(0,the);
#endif
      return;

}

void opentape()
{
  int i;

  if(tape_dev[scsi_id] == NULL) {
#ifdef VXWORKS
    if(iniscsi == 0){
      printf("Initialize exabyte %d\n",exbInit());
      iniscsi = 1;
    }
    sscanf(devname,"%d",&scsi_id);
    printf("Opening device %d\n",scsi_id);
    while((int)(tape_dev[scsi_id] = exbOpen(scsi_id)) == -1);
    printf("The tape dev %d is open %d\n",scsi_id,(int)tape_dev[scsi_id]);
#else
    scsi_id = -1;
    for(i=0;i<20;i++){
      if(tape_dev[i] == 0) break;
      if(tape_dev[i] != 0) {
	if(strcmp(devname,tapenam[i]) == 0) scsi_id = i;
      }
    }
    printf("Current id... %d\n",scsi_id);
    if(scsi_id >= 0) return;
    scsi_id  = i;
    strcpy(tapenam[i],devname);
    printf("Opening tapedrive %s\n",tapenam[i]);
    if((tape_dev[scsi_id] = open(tapenam[i],O_RDWR))<0) tape_dev[scsi_id] = 0;
#endif
  }
}

#ifndef VXWORKS
void SIGSEGV_handler()
{
   sleep(10);
   printf("In SIGSEGV handler...\n");
   printf("In SIGSEGV handler...\n");
   printf("In SIGSEGV handler...\n");
   printf("In SIGSEGV handler...\n");
   printf("In SIGSEGV handler...\n");
   sleep(10);
   ExitR();
}
#endif
void Process_Idle()
{
}
